#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars import _CustomIslamicHolidays
from holidays.calendars.gregorian import JAN, FEB, MAR, APR, MAY, JUN, JUL, AUG, SEP, OCT, NOV, DEC
from holidays.groups import (
    ChristianHolidays,
    InternationalHolidays,
    IslamicHolidays,
    StaticHolidays,
)
from holidays.observed_holiday_base import ObservedHolidayBase, SAT_SUN_TO_NEXT_WORKDAY


class Nigeria(
    ObservedHolidayBase, ChristianHolidays, InternationalHolidays, IslamicHolidays, StaticHolidays
):
    """Nigeria holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Nigeria>
        * [Public Holidays Decree 1979](https://web.archive.org/web/20240616072641/https://archive.gazettes.africa/archive/ng/1979/ng-government-gazette-supplement-dated-1979-02-01-no-5.pdf)
        * [Public Holidays (Amendment) Decree 1984](https://web.archive.org/web/20240615185836/https://archive.gazettes.africa/archive/ng/1984/ng-government-gazette-supplement-dated-1984-08-30-no-52-part-a.pdf)
        * [Public Holidays Act (2004 Consolidated Version)](https://web.archive.org/web/20250822094939/https://placng.org/lawsofnigeria/view2.php?sn=467)
        * <https://web.archive.org/web/20250829024504/https://fmino.gov.ng/may-29th-in-nigerias-history/>
        * <https://web.archive.org/web/20250829084254/https://www.iita.org/wp-content/uploads/2017/01/The-Bulletin-23-27-May-2011-No.-2070.pdf>
        * <https://web.archive.org/web/20170618031224/http://pulse.ng/local/inauguration-day-fg-declares-may-29-public-holiday-id3795830.html>
        * <https://web.archive.org/web/20250829081956/https://dailypost.ng/2019/05/27/fg-declares-may-29-public-holiday/>
        * <https://web.archive.org/web/20230620060859/https://interior.gov.ng/fg-declares-monday-may-29th-2023-work-free-day-for-presidential-inauguration/>
        * <https://web.archive.org/web/20250829060151/https://statehouse.gov.ng/news/president-buhari-declares-june-12-the-new-democracy-day/>
        * <https://web.archive.org/web/20250829085227/https://www.nairaland.com/4548139/may-29-remains-inauguration-day>
        * <https://web.archive.org/web/20250829030023/https://www.timeanddate.com/holidays/nigeria/2025?hol=9>
        * [2026 Eid al-Fitr](https://web.archive.org/web/20260324033902/https://nigeriahcottawa.ca/public-holidays/)

    In-lieu holidays have been in effect since at least 2010:
        * <http://archive.today/2025.08.30-142719/https://www.vanguardngr.com/2010/05/fg-declares-may-31-public-holiday-for-democracy/>
    """

    country = "NG"
    default_language = "en_NG"
    # %s (estimated).
    estimated_label = tr("%s (estimated)")
    # %s (observed, estimated).
    observed_estimated_label = tr("%s (observed, estimated)")
    # %s (observed).
    observed_label = tr("%s (observed)")
    # Public Holidays Decree 1979, in effect from January 1st, 1979.
    start_year = 1979
    supported_languages = ("en_NG", "en_US")

    def __init__(self, *args, islamic_show_estimated: bool = True, **kwargs):
        """
        Args:
            islamic_show_estimated:
                Whether to add "estimated" label to Islamic holidays name
                if holiday date is estimated.

        In Nigeria, the dates of the Islamic calendar usually fall a day later than
        the corresponding dates in the Umm al-Qura calendar.
        """
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        IslamicHolidays.__init__(
            self,
            cls=NigeriaIslamicHolidays,
            show_estimated=islamic_show_estimated,
            calendar_delta_days=+1,
        )
        StaticHolidays.__init__(self, NigeriaStaticHolidays)
        kwargs.setdefault("observed_rule", SAT_SUN_TO_NEXT_WORKDAY)
        kwargs.setdefault("observed_since", 2010)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        dts_observed = set()

        # New Year's Day.
        dts_observed.add(self._add_new_years_day(tr("New Year's Day")))

        # Good Friday.
        self._add_good_friday(tr("Good Friday"))

        # Easter Monday.
        self._add_easter_monday(tr("Easter Monday"))

        # Added via Public Holidays (Amendment) Decree 1984 on August 28th, 1984.
        if self._year >= 1985:
            # Workers' Day.
            dts_observed.add(self._add_labor_day(tr("Workers' Day")))

        # Added via Public Holidays (Amendment) Decree 2000.
        # Changed to June 12th for 2019 onwards on July 6th, 2018.
        if self._year >= 2000:
            # Democracy Day.
            name = tr("Democracy Day")
            dts_observed.add(
                self._add_holiday_jun_12(name)
                if self._year >= 2019
                else self._add_holiday_may_29(name)
            )

        if self._year >= 1999 and self._year % 4 == 3:
            # Presidential Inauguration Day.
            self._add_holiday_may_29(tr("Presidential Inauguration Day"))

        # National Day.
        dts_observed.add(self._add_holiday_oct_1(tr("National Day")))

        # Christmas Day.
        dts_observed.add(self._add_christmas_day(tr("Christmas Day")))

        # Boxing Day.
        dts_observed.add(self._add_christmas_day_two(tr("Boxing Day")))

        # Prophet's Birthday.
        dts_observed.update(self._add_mawlid_day(tr("Id el Maulud")))

        # Eid al-Fitr.
        dts_observed.update(self._add_eid_al_fitr_day(tr("Id el Fitr")))

        # Eid al-Fitr Holiday.
        dts_observed.update(self._add_eid_al_fitr_day_two(tr("Id el Fitr Holiday")))

        # Eid al-Adha.
        dts_observed.update(self._add_eid_al_adha_day(tr("Id el Kabir")))

        # Eid al-Adha Holiday.
        dts_observed.update(self._add_eid_al_adha_day_two(tr("Id el Kabir Holiday")))

        if self.observed:
            self._populate_observed(dts_observed)


class NG(Nigeria):
    pass


class NGA(Nigeria):
    pass


class NigeriaIslamicHolidays(_CustomIslamicHolidays):
    # https://web.archive.org/web/20250829051803/https://www.timeanddate.com/holidays/nigeria/id-el-kabir
    EID_AL_ADHA_DATES_CONFIRMED_YEARS = (1979, 2025)
    EID_AL_ADHA_DATES = {
        1982: (SEP, 29),
        1987: (AUG, 6),
        1988: (JUL, 25),
        1990: (JUL, 4),
        1992: (JUN, 11),
        1996: (APR, 29),
        2000: (MAR, 16),
        2005: (JAN, 21),
        2006: ((JAN, 10), (DEC, 31)),
        2007: (DEC, 19),
        2012: (OCT, 26),
        2013: (OCT, 15),
        2014: (OCT, 6),
        2020: (JUL, 31),
        2021: (JUL, 20),
        2022: (JUL, 9),
        2023: (JUN, 28),
        2024: (JUN, 16),
        2025: (JUN, 6),
    }

    # https://web.archive.org/web/20250829052410/https://www.timeanddate.com/holidays/nigeria/id-el-fitr
    EID_AL_FITR_DATES_CONFIRMED_YEARS = (1979, 2026)
    EID_AL_FITR_DATES = {
        1979: (AUG, 25),
        1982: (JUL, 23),
        1984: (JUN, 30),
        1987: (MAY, 30),
        1988: (MAY, 18),
        1992: (APR, 4),
        1996: (FEB, 21),
        2000: ((JAN, 8), (DEC, 28)),
        2004: (NOV, 14),
        2007: (OCT, 12),
        2008: (SEP, 29),
        2010: (SEP, 9),
        2011: (AUG, 30),
        2013: (AUG, 8),
        2014: (JUL, 28),
        2015: (JUL, 17),
        2016: (JUL, 6),
        2018: (JUN, 15),
        2019: (JUN, 4),
        2020: (MAY, 24),
        2021: (MAY, 13),
        2022: (MAY, 2),
        2023: (APR, 21),
        2024: (APR, 10),
        2025: (MAR, 30),
        2026: (MAR, 19),
    }

    # https://web.archive.org/web/20250829053435/https://www.timeanddate.com/holidays/nigeria/id-el-maulud
    # https://web.archive.org/web/20250829054319/https://pmnewsnigeria.com/2012/02/03/fg-declares-6-feb-public-holiday/
    MAWLID_DATES_CONFIRMED_YEARS = (1979, 2025)
    MAWLID_DATES = {
        1980: (JAN, 30),
        1981: (JAN, 18),
        1984: (DEC, 6),
        1989: (OCT, 13),
        1994: (AUG, 19),
        1997: (JUL, 18),
        1999: (JUN, 26),
        2001: (JUN, 4),
        2002: (MAY, 24),
        2005: (APR, 21),
        2007: (MAR, 31),
        2008: (MAR, 20),
        2009: (MAR, 9),
        2010: (FEB, 26),
        2012: (FEB, 6),
        2013: (JAN, 24),
        2014: (JAN, 13),
        2015: ((JAN, 2), (DEC, 24)),
        2018: (NOV, 20),
        2019: (NOV, 9),
        2020: (OCT, 29),
        2022: (OCT, 8),
        2023: (SEP, 27),
    }


class NigeriaStaticHolidays:
    """Nigeria special holidays.

    References:
        * <https://web.archive.org/web/20250829090324/https://www.pambazuka.org/governance/nigeria-holiday-blow-presidential-hopeful>
        * <https://web.archive.org/web/20250829084913/https://www.nbcnews.com/id/wbna36974655>
        * <https://web.archive.org/web/20250829081641/https://businessday.ng/uncategorized/article/2019-elections-fg-declares-friday-public-holiday-excludes-banksothers/>
        * <https://web.archive.org/web/20250829024209/https://statehouse.gov.ng/news/new-date-for-special-federal-executive-council-session-in-honour-of-president-muhammadu-buhari-to-be-announced/>
    """

    # Public Holiday for Elections.
    name_elections = tr("Public Holiday for Elections")

    special_public_holidays = {
        2007: (
            (APR, 12, name_elections),
            (APR, 13, name_elections),
        ),
        # Day of Mourning for President Umaru Yar'Adua.
        2010: (MAY, 6, tr("Day of Mourning for President Umaru Yar'Adua")),
        2019: (FEB, 22, name_elections),
        # Day of Mourning for President Muhammadu Buhari.
        2025: (JUL, 15, tr("Day of Mourning for President Muhammadu Buhari")),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.tonga import Tonga
from tests.common import CommonCountryTests


class TestTonga(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Tonga)

    def test_special_holidays(self):
        self.assertHoliday(
            "2017-11-29",
            "2019-09-19",
            "2019-11-15",
        )
        self.assertNoNonObservedHoliday(
            # 2021 Boxing Day
            "2021-12-27",
        )

    def test_new_years_day(self):
        name = "ʻUluaki ʻAho ʻo e Taʻu Foʻou"
        name_observed = f"{name} (fakatokangaʻi)"
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))
        obs_dts = (
            "1995-01-02",
            "2006-01-02",
            "2012-01-02",
        )
        self.assertHolidayName(name_observed, obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)
        self.assertNoHolidayName(name_observed, range(2017, self.end_year))

    def test_birthday_of_the_reigning_sovereign_of_tonga(self):
        name = "ʻAho ʻAloʻi ʻo ʻEne ʻAfio ko e Tuʻi ʻo Tonga ʻoku lolotonga Pule"
        self.assertHolidayName(
            name,
            (
                f"{year}-07-04"
                for year in (*range(self.start_year, 2007), *range(2013, self.end_year))
            ),
            (f"{year}-05-04" for year in range(2007, 2011)),
        )
        self.assertNoHolidayName(name, 2012)

        self.assertNoNonObservedHoliday(
            # Topou IV.
            "1993-07-05",
            "1999-07-05",
            "2004-07-05",
            # Topou V.
            "2008-05-05",
            # Topou V (Act 10 of 2010).
            "2011-05-02",
            # Topou VI.
            "2021-07-05",
            "2027-07-05",
        )
        self.assertNonObservedHoliday(
            # Topou V (Act 10 of 2010).
            "2011-05-04",
        )

    def test_birthday_of_the_heir_to_the_crown_of_tonga(self):
        name = "ʻAho ʻAloʻi ʻo e ʻEa ki he Kalauni ʻo Tonga"
        self.assertHolidayName(
            name,
            (f"{year}-05-04" for year in range(self.start_year, 2007)),
            (f"{year}-07-12" for year in range(2007, 2011)),
            (f"{year}-09-17" for year in range(2012, self.end_year)),
        )

        self.assertNoNonObservedHoliday(
            # Topou IV's Heir: Topou V.
            "1997-05-05",
            "2003-05-05",
            # Topou V's Heir: Topou VI.
            "2009-07-13",
            # Topou V's Heir: Topou VI (Act 10 of 2010).
            "2011-07-11",
            # Topou VI's Heir: Tupoutoʻa ʻUlukalala.
            "2017-09-18",
            "2023-09-18",
            "2028-09-18",
        )
        self.assertNonObservedHoliday(
            # Topou V's Heir: Topou VI (Act 10 of 2010).
            "2011-07-12",
        )

    def test_good_friday(self):
        name = "Falaite Lelei"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_monday(self):
        name = "Monite ʻo e Toetuʻu"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_anzac_day(self):
        name = "ʻAho Anzac"
        name_observed = f"{name} (fakatokangaʻi)"
        self.assertHolidayName(name, (f"{year}-04-25" for year in self.full_range))
        obs_dts = (
            "1999-04-26",
            "2004-04-26",
            "2010-04-26",
        )
        self.assertHolidayName(name_observed, obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)
        self.assertNoHolidayName(name_observed, range(2017, self.end_year))

    def test_emancipation_day(self):
        name = "ʻAho Tauʻataina"
        self.assertHolidayName(name, (f"{year}-06-04" for year in range(self.start_year, 2010)))
        self.assertHoliday(
            "2020-06-08",
            "2021-06-07",
            "2022-06-06",
            "2023-06-05",
            "2024-06-03",
            "2025-06-02",
        )
        self.assertNonObservedHolidayName(name, range(2010, self.end_year))

        obs_dts = (
            "1995-06-05",
            "2000-06-05",
            "2006-06-05",
        )
        self.assertHolidayName(f"{name} (fakatokangaʻi)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_anniversary_of_the_coronation_day_of_the_reigning_sovereign_of_tonga(self):
        name = (
            "Fakamanatu ʻo e ʻAho Hilifaki Kalauni ʻo ʻEne ʻAfio ko e Tuʻi ʻo Tonga "
            "ʻa ia ʻoku lolotonga Pule"
        )
        self.assertHolidayName(name, (f"{year}-08-01" for year in {2008, 2009, 2011}))
        self.assertNoHolidayName(name, range(self.start_year, 2008), range(2012, self.end_year))

        self.assertNoNonObservedHoliday(
            # Topou V (Act 10 of 2010).
            "2010-08-02",
        )
        self.assertNonObservedHoliday(
            # Topou V (Act 10 of 2010).
            "2010-08-01",
        )

    def test_constitution_day(self):
        name = "ʻAho Konisitutone"
        self.assertHolidayName(name, (f"{year}-11-04" for year in range(self.start_year, 2010)))
        self.assertHoliday(
            "2020-11-02",
            "2021-11-08",
            "2022-11-07",
            "2023-11-06",
            "2024-11-04",
            "2025-11-03",
        )
        self.assertNonObservedHolidayName(name, range(2010, self.end_year))

        obs_dts = (
            "1990-11-05",
            "2001-11-05",
            "2007-11-05",
        )
        self.assertHolidayName(f"{name} (fakatokangaʻi)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_anniversary_of_the_coronation_of_hm_king_george_tupou_i(self):
        name = "ʻAho Fakamanatu ʻo e Hilifaki Kalauni ʻo ʻEne ʻAfio ko Siaosi Tupou I"
        self.assertHolidayName(name, (f"{year}-12-04" for year in range(self.start_year, 2010)))
        self.assertHoliday(
            "2020-12-07",
            "2021-12-06",
            "2022-12-05",
            "2023-12-04",
            "2024-12-02",
            "2025-12-08",
        )
        self.assertNonObservedHolidayName(name, range(2010, self.end_year))

        obs_dts = (
            "1994-12-05",
            "2005-12-05",
        )
        self.assertHolidayName(f"{name} (fakatokangaʻi)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_christmas_day(self):
        self.assertHolidayName("ʻAho Kilisimasi", (f"{year}-12-25" for year in self.full_range))

    def test_boxing_day(self):
        name = "ʻAho 2 ʻo e Kilisimasi"
        name_observed = f"{name} (fakatokangaʻi)"
        self.assertHolidayName(name, (f"{year}-12-26" for year in self.full_range))
        obs_dts = (
            "1994-12-27",
            "2005-12-27",
            "2021-12-27",
        )
        self.assertHolidayName(name_observed, obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)
        self.assertNoHolidayName(name_observed, range(2009, 2021), range(2022, self.end_year))

    def test_2017(self):
        # https://www.officeholidays.com/countries/tonga/2017
        self.assertHolidaysInYear(
            2017,
            ("2017-01-01", "ʻUluaki ʻAho ʻo e Taʻu Foʻou"),
            ("2017-04-14", "Falaite Lelei"),
            ("2017-04-17", "Monite ʻo e Toetuʻu"),
            ("2017-04-25", "ʻAho Anzac"),
            ("2017-06-05", "ʻAho Tauʻataina (fakatokangaʻi)"),
            ("2017-07-04", "ʻAho ʻAloʻi ʻo ʻEne ʻAfio ko e Tuʻi ʻo Tonga ʻoku lolotonga Pule"),
            ("2017-09-17", "ʻAho ʻAloʻi ʻo e ʻEa ki he Kalauni ʻo Tonga"),
            ("2017-09-18", "ʻAho ʻAloʻi ʻo e ʻEa ki he Kalauni ʻo Tonga (fakatokangaʻi)"),
            ("2017-11-06", "ʻAho Konisitutone (fakatokangaʻi)"),
            ("2017-11-29", "ʻAho malolo ʻakapulu ʻa Tonga"),
            (
                "2017-12-04",
                "ʻAho Fakamanatu ʻo e Hilifaki Kalauni ʻo ʻEne ʻAfio ko Siaosi Tupou I",
            ),
            ("2017-12-25", "ʻAho Kilisimasi"),
            ("2017-12-26", "ʻAho 2 ʻo e Kilisimasi"),
        )

    def test_2018(self):
        # https://www.gov.to/press-release/tonga-public-holidays-for-2018/
        self.assertHolidaysInYear(
            2018,
            ("2018-01-01", "ʻUluaki ʻAho ʻo e Taʻu Foʻou"),
            ("2018-03-30", "Falaite Lelei"),
            ("2018-04-02", "Monite ʻo e Toetuʻu"),
            ("2018-04-25", "ʻAho Anzac"),
            ("2018-06-04", "ʻAho Tauʻataina"),
            ("2018-07-04", "ʻAho ʻAloʻi ʻo ʻEne ʻAfio ko e Tuʻi ʻo Tonga ʻoku lolotonga Pule"),
            ("2018-09-17", "ʻAho ʻAloʻi ʻo e ʻEa ki he Kalauni ʻo Tonga"),
            ("2018-11-05", "ʻAho Konisitutone (fakatokangaʻi)"),
            (
                "2018-12-03",
                "ʻAho Fakamanatu ʻo e Hilifaki Kalauni ʻo ʻEne ʻAfio ko "
                "Siaosi Tupou I (fakatokangaʻi)",
            ),
            ("2018-12-25", "ʻAho Kilisimasi"),
            ("2018-12-26", "ʻAho 2 ʻo e Kilisimasi"),
        )

    def test_2019(self):
        # https://www.gov.to/press-release/tonga-public-holidays-for-2019-2/
        self.assertHolidaysInYear(
            2019,
            ("2019-01-01", "ʻUluaki ʻAho ʻo e Taʻu Foʻou"),
            ("2019-04-19", "Falaite Lelei"),
            ("2019-04-22", "Monite ʻo e Toetuʻu"),
            ("2019-04-25", "ʻAho Anzac"),
            ("2019-06-03", "ʻAho Tauʻataina (fakatokangaʻi)"),
            ("2019-07-04", "ʻAho ʻAloʻi ʻo ʻEne ʻAfio ko e Tuʻi ʻo Tonga ʻoku lolotonga Pule"),
            ("2019-09-17", "ʻAho ʻAloʻi ʻo e ʻEa ki he Kalauni ʻo Tonga"),
            ("2019-09-19", "Meʻafakaʻeiki ʻo e Siteiti ʻAkilisi Pōhiva"),
            ("2019-11-04", "ʻAho Konisitutone"),
            ("2019-11-15", "ʻAho malolo ʻakapulu ʻa Tonga"),
            (
                "2019-12-02",
                "ʻAho Fakamanatu ʻo e Hilifaki Kalauni ʻo ʻEne ʻAfio ko "
                "Siaosi Tupou I (fakatokangaʻi)",
            ),
            ("2019-12-25", "ʻAho Kilisimasi"),
            ("2019-12-26", "ʻAho 2 ʻo e Kilisimasi"),
        )

    def test_2020(self):
        # https://www.gov.to/press-release/tonga-public-holidays-for-2020/
        self.assertHolidaysInYear(
            2020,
            ("2020-01-01", "ʻUluaki ʻAho ʻo e Taʻu Foʻou"),
            ("2020-04-10", "Falaite Lelei"),
            ("2020-04-13", "Monite ʻo e Toetuʻu"),
            ("2020-04-25", "ʻAho Anzac"),
            ("2020-06-08", "ʻAho Tauʻataina (fakatokangaʻi)"),
            ("2020-07-04", "ʻAho ʻAloʻi ʻo ʻEne ʻAfio ko e Tuʻi ʻo Tonga ʻoku lolotonga Pule"),
            ("2020-09-17", "ʻAho ʻAloʻi ʻo e ʻEa ki he Kalauni ʻo Tonga"),
            ("2020-11-02", "ʻAho Konisitutone (fakatokangaʻi)"),
            (
                "2020-12-07",
                "ʻAho Fakamanatu ʻo e Hilifaki Kalauni ʻo ʻEne ʻAfio ko "
                "Siaosi Tupou I (fakatokangaʻi)",
            ),
            ("2020-12-25", "ʻAho Kilisimasi"),
            ("2020-12-26", "ʻAho 2 ʻo e Kilisimasi"),
        )

    def test_2021(self):
        # https://www.gov.to/press-release/tonga-public-holidays-for-2021/
        self.assertHolidaysInYear(
            2021,
            ("2021-01-01", "ʻUluaki ʻAho ʻo e Taʻu Foʻou"),
            ("2021-04-02", "Falaite Lelei"),
            ("2021-04-05", "Monite ʻo e Toetuʻu"),
            ("2021-04-25", "ʻAho Anzac"),
            ("2021-06-07", "ʻAho Tauʻataina (fakatokangaʻi)"),
            ("2021-07-04", "ʻAho ʻAloʻi ʻo ʻEne ʻAfio ko e Tuʻi ʻo Tonga ʻoku lolotonga Pule"),
            (
                "2021-07-05",
                "ʻAho ʻAloʻi ʻo ʻEne ʻAfio ko e Tuʻi ʻo Tonga ʻoku lolotonga Pule (fakatokangaʻi)",
            ),
            ("2021-09-17", "ʻAho ʻAloʻi ʻo e ʻEa ki he Kalauni ʻo Tonga"),
            ("2021-11-08", "ʻAho Konisitutone (fakatokangaʻi)"),
            (
                "2021-12-06",
                "ʻAho Fakamanatu ʻo e Hilifaki Kalauni ʻo ʻEne ʻAfio ko "
                "Siaosi Tupou I (fakatokangaʻi)",
            ),
            ("2021-12-25", "ʻAho Kilisimasi"),
            ("2021-12-26", "ʻAho 2 ʻo e Kilisimasi"),
            ("2021-12-27", "ʻAho 2 ʻo e Kilisimasi (fakatokangaʻi)"),  # ???
        )

    def test_2022(self):
        # https://www.gov.to/press-release/tonga-public-holidays-for-2022/
        self.assertHolidaysInYear(
            2022,
            ("2022-01-01", "ʻUluaki ʻAho ʻo e Taʻu Foʻou"),
            ("2022-04-15", "Falaite Lelei"),
            ("2022-04-18", "Monite ʻo e Toetuʻu"),
            ("2022-04-25", "ʻAho Anzac"),
            ("2022-06-06", "ʻAho Tauʻataina (fakatokangaʻi)"),
            ("2022-07-04", "ʻAho ʻAloʻi ʻo ʻEne ʻAfio ko e Tuʻi ʻo Tonga ʻoku lolotonga Pule"),
            ("2022-09-17", "ʻAho ʻAloʻi ʻo e ʻEa ki he Kalauni ʻo Tonga"),
            ("2022-11-07", "ʻAho Konisitutone (fakatokangaʻi)"),
            (
                "2022-12-05",
                "ʻAho Fakamanatu ʻo e Hilifaki Kalauni ʻo ʻEne ʻAfio ko "
                "Siaosi Tupou I (fakatokangaʻi)",
            ),
            ("2022-12-25", "ʻAho Kilisimasi"),
            ("2022-12-26", "ʻAho 2 ʻo e Kilisimasi"),
        )

    def test_2024(self):
        # https://www.gov.to/press-release/tonga-public-holidays-for-2024/
        self.assertHolidaysInYear(
            2024,
            ("2024-01-01", "ʻUluaki ʻAho ʻo e Taʻu Foʻou"),
            ("2024-03-29", "Falaite Lelei"),
            ("2024-04-01", "Monite ʻo e Toetuʻu"),
            ("2024-04-25", "ʻAho Anzac"),
            ("2024-06-03", "ʻAho Tauʻataina (fakatokangaʻi)"),
            ("2024-07-04", "ʻAho ʻAloʻi ʻo ʻEne ʻAfio ko e Tuʻi ʻo Tonga ʻoku lolotonga Pule"),
            ("2024-09-17", "ʻAho ʻAloʻi ʻo e ʻEa ki he Kalauni ʻo Tonga"),
            ("2024-11-04", "ʻAho Konisitutone"),
            (
                "2024-12-02",
                "ʻAho Fakamanatu ʻo e Hilifaki Kalauni ʻo ʻEne ʻAfio ko "
                "Siaosi Tupou I (fakatokangaʻi)",
            ),
            ("2024-12-25", "ʻAho Kilisimasi"),
            ("2024-12-26", "ʻAho 2 ʻo e Kilisimasi"),
        )

    def test_l10n_default(self):
        # https://www.gov.to/press-release/tonga-public-holidays-for-2023/
        self.assertLocalizedHolidays(
            ("2023-01-01", "ʻUluaki ʻAho ʻo e Taʻu Foʻou"),
            ("2023-04-07", "Falaite Lelei"),
            ("2023-04-10", "Monite ʻo e Toetuʻu"),
            ("2023-04-25", "ʻAho Anzac"),
            ("2023-06-05", "ʻAho Tauʻataina (fakatokangaʻi)"),
            ("2023-07-04", "ʻAho ʻAloʻi ʻo ʻEne ʻAfio ko e Tuʻi ʻo Tonga ʻoku lolotonga Pule"),
            ("2023-09-17", "ʻAho ʻAloʻi ʻo e ʻEa ki he Kalauni ʻo Tonga"),
            ("2023-09-18", "ʻAho ʻAloʻi ʻo e ʻEa ki he Kalauni ʻo Tonga (fakatokangaʻi)"),
            ("2023-11-06", "ʻAho Konisitutone (fakatokangaʻi)"),
            (
                "2023-12-04",
                "ʻAho Fakamanatu ʻo e Hilifaki Kalauni ʻo ʻEne ʻAfio ko Siaosi Tupou I",
            ),
            ("2023-12-25", "ʻAho Kilisimasi"),
            ("2023-12-26", "ʻAho 2 ʻo e Kilisimasi"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2023-01-01", "New Year's Day"),
            ("2023-04-07", "Good Friday"),
            ("2023-04-10", "Easter Monday"),
            ("2023-04-25", "Anzac Day"),
            ("2023-06-05", "Emancipation Day (observed)"),
            ("2023-07-04", "Birthday of the Reigning Sovereign of Tonga"),
            ("2023-09-17", "Birthday of the Heir to the Crown of Tonga"),
            ("2023-09-18", "Birthday of the Heir to the Crown of Tonga (observed)"),
            ("2023-11-06", "Constitution Day (observed)"),
            ("2023-12-04", "Anniversary of the Coronation of HM King George Tupou I"),
            ("2023-12-25", "Christmas Day"),
            ("2023-12-26", "Boxing Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.turks_and_caicos_islands import TurksAndCaicosIslands
from tests.common import CommonCountryTests


class TestTurksAndCaicosIslands(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(TurksAndCaicosIslands)

    def test_new_years_day(self):
        name = "New Year's Day"
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))
        obs_dts = (
            "2000-01-03",
            "2005-01-03",
            "2006-01-02",
            "2011-01-03",
            "2012-01-02",
            "2017-01-02",
            "2022-01-03",
            "2023-01-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_commonwealth_day(self):
        name = "Commonwealth Day"
        self.assertHolidayName(
            name,
            "2020-03-09",
            "2021-03-08",
            "2022-03-14",
            "2023-03-13",
            "2024-03-11",
            "2025-03-10",
        )
        self.assertHolidayName(name, self.full_range)

    def test_good_friday(self):
        name = "Good Friday"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_monday(self):
        name = "Easter Monday"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_jags_mccartney_day(self):
        name_pre_2020 = "National Heroes Day"
        name_2020 = "JAGS McCartney Day"

        self.assertHolidayName(
            name_pre_2020,
            "2014-05-26",
            "2015-05-25",
            "2016-05-30",
            "2017-05-29",
            "2018-05-28",
            "2019-05-27",
        )
        self.assertHolidayName(
            name_2020,
            "2020-05-25",
            "2021-05-31",
            "2022-05-30",
            "2023-05-29",
            "2024-05-27",
            "2025-05-26",
        )

        self.assertHolidayName(name_pre_2020, range(self.start_year, 2020))
        self.assertHolidayName(name_2020, range(2020, self.end_year))
        self.assertNoHolidayName(name_pre_2020, range(2020, self.end_year))
        self.assertNoHolidayName(name_2020, range(self.start_year, 2020))

    def test_sovereign_birthday(self):
        name_pre_2023 = "Queen's Birthday"
        name_2023 = "King's Birthday"

        self.assertHolidayName(
            name_pre_2023,
            "2017-06-12",
            "2018-06-11",
            "2019-06-10",
            "2020-06-08",
            "2021-06-14",
            "2022-06-03",
        )
        self.assertHolidayName(
            name_2023,
            "2023-06-19",
            "2024-06-17",
            "2025-06-23",
        )

        self.assertHolidayName(name_pre_2023, range(self.start_year, 2023))
        self.assertHolidayName(name_2023, range(2023, self.end_year))
        self.assertNoHolidayName(name_pre_2023, range(2023, self.end_year))
        self.assertNoHolidayName(name_2023, range(self.start_year, 2023))

    def test_emancipation_day(self):
        name = "Emancipation Day"
        self.assertHolidayName(name, (f"{year}-08-01" for year in self.full_range))
        obs_dts = (
            "2004-08-02",
            "2009-08-03",
            "2010-08-02",
            "2015-08-03",
            "2020-08-03",
            "2021-08-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_national_youth_day(self):
        name = "National Youth Day"
        self.assertHolidayName(
            name,
            "2020-09-25",
            "2021-09-24",
            "2022-09-30",
            "2023-09-29",
            "2024-09-27",
            "2025-09-26",
        )
        self.assertHolidayName(name, self.full_range)

    def test_national_heritage_day(self):
        name_pre_2014 = "Columbus Day"
        name_2014 = "National Heritage Day"

        self.assertHolidayName(
            name_pre_2014,
            "2010-10-11",
            "2011-10-10",
            "2012-10-08",
            "2013-10-14",
        )
        self.assertHolidayName(
            name_2014,
            "2014-10-13",
            "2015-10-12",
            "2016-10-10",
            "2017-10-09",
            "2018-10-08",
            "2019-10-14",
        )

        self.assertHolidayName(name_pre_2014, range(self.start_year, 2014))
        self.assertHolidayName(name_2014, range(2014, self.end_year))
        self.assertNoHolidayName(name_pre_2014, range(2014, self.end_year))
        self.assertNoHolidayName(name_2014, range(self.start_year, 2014))

    def test_national_day_of_thanksgiving(self):
        name = "National Day of Thanksgiving"
        self.assertHolidayName(
            name,
            "2020-11-27",
            "2021-11-26",
            "2022-11-25",
            "2023-11-24",
            "2024-11-22",
            "2025-11-28",
            "2026-08-30",
            "2027-08-29",
            "2028-08-27",
        )
        self.assertHolidayName(name, range(2014, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 2014))

    def test_constitution_day(self):
        name = "Constitution Day"
        self.assertHolidayName(
            name,
            "2026-08-31",
            "2027-08-30",
            "2028-08-28",
            "2029-08-27",
            "2030-08-26",
        )
        self.assertHolidayName(name, range(2026, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 2026))

    def test_christmas_day(self):
        name = "Christmas Day"
        self.assertHolidayName(name, (f"{year}-12-25" for year in self.full_range))
        obs_dts = (
            "2004-12-27",
            "2005-12-27",
            "2010-12-27",
            "2011-12-27",
            "2016-12-27",
            "2021-12-27",
            "2022-12-27",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_boxing_day(self):
        name = "Boxing Day"
        self.assertHolidayName(name, (f"{year}-12-26" for year in self.full_range))
        obs_dts = (
            "2004-12-28",
            "2009-12-28",
            "2010-12-28",
            "2015-12-28",
            "2020-12-28",
            "2021-12-28",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_2023(self):
        self.assertHolidaysInYear(
            2023,
            ("2023-01-01", "New Year's Day"),
            ("2023-01-02", "New Year's Day (observed)"),
            ("2023-03-13", "Commonwealth Day"),
            ("2023-04-07", "Good Friday"),
            ("2023-04-10", "Easter Monday"),
            ("2023-05-29", "JAGS McCartney Day"),
            ("2023-06-19", "King's Birthday"),
            ("2023-08-01", "Emancipation Day"),
            ("2023-09-29", "National Youth Day"),
            ("2023-10-09", "National Heritage Day"),
            ("2023-11-24", "National Day of Thanksgiving"),
            ("2023-12-25", "Christmas Day"),
            ("2023-12-26", "Boxing Day"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2025-01-01", "New Year's Day"),
            ("2025-03-10", "Commonwealth Day"),
            ("2025-04-18", "Good Friday"),
            ("2025-04-21", "Easter Monday"),
            ("2025-05-26", "JAGS McCartney Day"),
            ("2025-06-23", "King's Birthday"),
            ("2025-08-01", "Emancipation Day"),
            ("2025-09-26", "National Youth Day"),
            ("2025-10-13", "National Heritage Day"),
            ("2025-11-28", "National Day of Thanksgiving"),
            ("2025-12-25", "Christmas Day"),
            ("2025-12-26", "Boxing Day"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2025-01-01", "New Year's Day"),
            ("2025-03-10", "Commonwealth Day"),
            ("2025-04-18", "Good Friday"),
            ("2025-04-21", "Easter Monday"),
            ("2025-05-26", "JAGS McCartney Day"),
            ("2025-06-23", "King's Birthday"),
            ("2025-08-01", "Emancipation Day"),
            ("2025-09-26", "National Youth Day"),
            ("2025-10-13", "National Heritage Day"),
            ("2025-11-28", "National Day of Thanksgiving"),
            ("2025-12-25", "Christmas Day"),
            ("2025-12-26", "Boxing Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.constants import WORKDAY
from holidays.countries.fiji import Fiji
from tests.common import CommonCountryTests


class TestFiji(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Fiji)

    def test_no_holidays(self):
        super().test_no_holidays()

        self.assertNoHolidays(Fiji(categories=WORKDAY, years=range(self.start_year, 2023)))

    def test_new_years_day(self):
        name = "New Year's Day"
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))

        dt_obs = (
            "2017-01-02",
            "2022-01-03",
            "2023-01-02",
        )
        self.assertHolidayName(f"{name} (observed)", dt_obs)
        self.assertNoNonObservedHoliday(dt_obs)

    def test_good_friday(self):
        name = "Good Friday"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_saturday(self):
        name = "Easter Saturday"
        self.assertHolidayName(
            name,
            "2020-04-11",
            "2021-04-03",
            "2022-04-16",
            "2023-04-08",
            "2024-03-30",
            "2025-04-19",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_monday(self):
        name = "Easter Monday"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_girmit_day(self):
        name = "Girmit Day"
        self.assertHolidayName(
            name,
            "2023-05-15",
            "2024-05-13",
            "2025-05-12",
        )
        self.assertHolidayName(name, range(2023, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 2023))

    def test_national_sports_day(self):
        name = "National Sports Day"
        self.assertHolidayName(
            name,
            "2016-06-24",
            "2017-06-30",
            "2018-06-29",
        )
        self.assertNoHolidayName(name, range(2019, self.end_year))

    def test_constitution_day(self):
        name = "Constitution Day"

        # Public Holidays
        self.assertHolidayName(name, (f"{year}-09-07" for year in range(self.start_year, 2023)))
        self.assertNoHolidayName(name, range(2023, self.end_year))

        dt_obs = ("2019-09-09",)
        self.assertHolidayName(f"{name} (observed)", dt_obs)
        self.assertNoNonObservedHoliday(dt_obs)

        # Workdays.
        self.assertWorkdayHolidayName(
            name, (f"{year}-09-07" for year in range(2023, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 2023))

    def test_ratu_sir_lala_sukuna_day(self):
        name = "Ratu Sir Lala Sukuna Day"
        self.assertHolidayName(
            name,
            "2023-05-29",
            "2024-05-31",
            "2025-05-30",
        )
        self.assertHolidayName(name, range(2023, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 2023))

    def test_fiji_day(self):
        self.assertHolidayName("Fiji Day", (f"{year}-10-10" for year in self.full_range))

    def test_diwali(self):
        name = "Diwali"
        self.assertHolidayName(
            name,
            "2016-10-31",
            "2017-10-19",
            "2018-11-07",
            "2019-10-28",
            "2020-11-14",
            "2021-11-04",
            "2022-10-25",
            "2023-11-13",
            "2024-11-01",
            "2025-10-21",
        )

        dt_obs = ("2020-11-16",)
        self.assertHolidayName(f"{name} (observed)", dt_obs)
        self.assertNoNonObservedHoliday(dt_obs)

    def test_christmas_day(self):
        name = "Christmas Day"
        self.assertHolidayName(name, (f"{year}-12-25" for year in self.full_range))

        dt_obs = (
            "2016-12-27",
            "2021-12-27",
            "2022-12-27",
        )
        self.assertHolidayName(f"{name} (observed)", dt_obs)
        self.assertNoNonObservedHoliday(dt_obs)

    def test_boxing_day(self):
        name = "Boxing Day"
        self.assertHolidayName(name, (f"{year}-12-26" for year in self.full_range))

        dt_obs = (
            "2020-12-28",
            "2021-12-28",
            "2026-12-28",
            "2027-12-28",
        )
        self.assertHolidayName(f"{name} (observed)", dt_obs)
        self.assertNoNonObservedHoliday(dt_obs)

    def test_prophets_birthday(self):
        name = "Prophet Mohammed's Birthday"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-10-31",
            "2021-10-18",
            "2022-10-07",
            "2023-09-30",
            "2024-09-16",
            "2025-09-06",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

        dt_obs = (
            "2019-11-11",
            "2020-11-02",
            "2023-10-02",
            "2025-09-08",
        )
        self.assertHolidayName(f"{name} (observed)", dt_obs)
        self.assertNoNonObservedHoliday(dt_obs)

    def test_2019(self):
        # https://web.archive.org/web/20191018023027/https://www.fiji.gov.fj/About-Fiji/Public-Holidays
        self.assertHolidaysInYear(
            2019,
            ("2019-01-01", "New Year's Day"),
            ("2019-04-19", "Good Friday"),
            ("2019-04-20", "Easter Saturday"),
            ("2019-04-22", "Easter Monday"),
            ("2019-09-07", "Constitution Day"),
            ("2019-09-09", "Constitution Day (observed)"),
            ("2019-10-10", "Fiji Day"),
            ("2019-10-28", "Diwali"),
            ("2019-11-09", "Prophet Mohammed's Birthday"),
            ("2019-11-11", "Prophet Mohammed's Birthday (observed)"),
            ("2019-12-25", "Christmas Day"),
            ("2019-12-26", "Boxing Day"),
        )

    def test_2020(self):
        # https://web.archive.org/web/20210103183942/https://www.fiji.gov.fj/About-Fiji/Public-Holidays
        self.assertHolidaysInYear(
            2020,
            ("2020-01-01", "New Year's Day"),
            ("2020-04-10", "Good Friday"),
            ("2020-04-11", "Easter Saturday"),
            ("2020-04-13", "Easter Monday"),
            ("2020-09-07", "Constitution Day"),
            ("2020-10-10", "Fiji Day"),
            ("2020-10-31", "Prophet Mohammed's Birthday"),
            ("2020-11-02", "Prophet Mohammed's Birthday (observed)"),
            ("2020-11-14", "Diwali"),
            ("2020-11-16", "Diwali (observed)"),
            ("2020-12-25", "Christmas Day"),
            ("2020-12-26", "Boxing Day"),
            ("2020-12-28", "Boxing Day (observed)"),
        )

    def test_2021(self):
        # https://web.archive.org/web/20221223004409/https://www.fiji.gov.fj/About-Fiji/Public-Holidays
        self.assertHolidaysInYear(
            2021,
            ("2021-01-01", "New Year's Day"),
            ("2021-04-02", "Good Friday"),
            ("2021-04-03", "Easter Saturday"),
            ("2021-04-05", "Easter Monday"),
            ("2021-09-07", "Constitution Day"),
            ("2021-10-10", "Fiji Day"),
            ("2021-10-18", "Prophet Mohammed's Birthday"),
            ("2021-11-04", "Diwali"),
            ("2021-12-25", "Christmas Day"),
            ("2021-12-26", "Boxing Day"),
            ("2021-12-27", "Christmas Day (observed)"),
            ("2021-12-28", "Boxing Day (observed)"),
        )

    def test_2022(self):
        # https://web.archive.org/web/20221223004409/https://www.fiji.gov.fj/About-Fiji/Public-Holidays
        self.assertHolidaysInYear(
            2022,
            ("2022-01-01", "New Year's Day"),
            ("2022-01-03", "New Year's Day (observed)"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-16", "Easter Saturday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-09-07", "Constitution Day"),
            ("2022-10-07", "Prophet Mohammed's Birthday"),
            ("2022-10-10", "Fiji Day"),
            ("2022-10-25", "Diwali"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Boxing Day"),
            ("2022-12-27", "Christmas Day (observed)"),
        )

    def test_2023(self):
        # https://web.archive.org/web/20231129154609/https://www.fiji.gov.fj/About-Fiji/Public-Holidays
        self.assertHolidaysInYear(
            2023,
            ("2023-01-01", "New Year's Day"),
            ("2023-01-02", "New Year's Day (observed)"),
            ("2023-04-07", "Good Friday"),
            ("2023-04-08", "Easter Saturday"),
            ("2023-04-10", "Easter Monday"),
            ("2023-05-15", "Girmit Day"),
            ("2023-05-29", "Ratu Sir Lala Sukuna Day"),
            ("2023-09-30", "Prophet Mohammed's Birthday"),
            ("2023-10-02", "Prophet Mohammed's Birthday (observed)"),
            ("2023-10-10", "Fiji Day"),
            ("2023-11-13", "Diwali"),
            ("2023-12-25", "Christmas Day"),
            ("2023-12-26", "Boxing Day"),
        )

    def test_2024(self):
        # https://web.archive.org/web/20250121185434/https://www.fiji.gov.fj/About-Fiji/Public-Holidays
        self.assertHolidaysInYear(
            2024,
            ("2024-01-01", "New Year's Day"),
            ("2024-03-29", "Good Friday"),
            ("2024-03-30", "Easter Saturday"),
            ("2024-04-01", "Easter Monday"),
            ("2024-05-13", "Girmit Day"),
            ("2024-05-31", "Ratu Sir Lala Sukuna Day"),
            ("2024-09-16", "Prophet Mohammed's Birthday"),
            ("2024-10-10", "Fiji Day"),
            ("2024-11-01", "Diwali"),
            ("2024-12-25", "Christmas Day"),
            ("2024-12-26", "Boxing Day"),
        )

    def test_2025(self):
        # https://web.archive.org/web/20250121185434/https://www.fiji.gov.fj/About-Fiji/Public-Holidays
        self.assertHolidaysInYear(
            2025,
            ("2025-01-01", "New Year's Day"),
            ("2025-04-18", "Good Friday"),
            ("2025-04-19", "Easter Saturday"),
            ("2025-04-21", "Easter Monday"),
            ("2025-05-12", "Girmit Day"),
            ("2025-05-30", "Ratu Sir Lala Sukuna Day"),
            ("2025-09-06", "Prophet Mohammed's Birthday"),
            ("2025-09-08", "Prophet Mohammed's Birthday (observed)"),
            ("2025-10-10", "Fiji Day"),
            ("2025-10-21", "Diwali"),
            ("2025-12-25", "Christmas Day"),
            ("2025-12-26", "Boxing Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.niger import Niger
from tests.common import CommonCountryTests


class TestNiger(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Niger)

    def test_new_years_day(self):
        name = "Jour de l'An"
        name_observed = f"{name} (observé)"
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))
        obs_dts = (
            "2012-01-02",
            "2017-01-02",
            "2023-01-02",
        )
        self.assertHolidayName(name_observed, obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)
        self.assertNoHolidayName(name_observed, range(self.start_year, 1998))

    def test_easter_monday(self):
        name = "Lundi de Pâques"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_concord_day(self):
        name = "Fête nationale de la Concorde"
        name_observed = f"{name} (observé)"
        self.assertHolidayName(name, (f"{year}-04-24" for year in range(1995, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1995))
        obs_dts = (
            "2011-04-25",
            "2016-04-25",
            "2022-04-25",
        )
        self.assertHolidayName(name_observed, obs_dts)
        self.assertNoNonObservedHolidayName(name, obs_dts)
        self.assertNoHolidayName(name_observed, range(self.start_year, 1998))

    def test_international_labor_day(self):
        name = "Journée internationale du travail"
        name_observed = f"{name} (observé)"
        self.assertHolidayName(name, (f"{year}-05-01" for year in self.full_range))
        obs_dts = (
            "2011-05-02",
            "2016-05-02",
            "2022-05-02",
        )
        self.assertHolidayName(name_observed, obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)
        self.assertNoHolidayName(name_observed, range(self.start_year, 1998))

    def test_ascension_day(self):
        name = "Ascension"
        self.assertNoHolidayName(name)
        self.assertOptionalHolidayName(
            name,
            "2020-05-21",
            "2021-05-13",
            "2022-05-26",
            "2023-05-18",
            "2024-05-09",
            "2025-05-29",
        )
        self.assertOptionalHolidayName(name, self.full_range)

    def test_whit_monday(self):
        name = "Lundi de Pentecôte"
        self.assertNoHolidayName(name)
        self.assertOptionalHolidayName(
            name,
            "2020-06-01",
            "2021-05-24",
            "2022-06-06",
            "2023-05-29",
            "2024-05-20",
            "2025-06-09",
        )
        self.assertOptionalHolidayName(name, self.full_range)

    def test_cnsp_coup_anniversary(self):
        name = "Anniversaire du coup d'État du CNSP"
        name_observed = f"{name} (observé)"
        self.assertHolidayName(name, (f"{year}-07-26" for year in range(2024, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2024))
        obs_dts = (
            "2026-07-27",
            "2037-07-27",
        )
        self.assertHolidayName(name_observed, obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)
        self.assertNoHolidayName(name_observed, range(self.start_year, 2024))

    def test_independence_day(self):
        name_1960 = "Jour de l'indépendance"
        name_1961 = "L'anniversaire de la proclamation de l'indépendance"
        name_observed = f"{name_1961} (observé)"
        self.assertHolidayName(name_1960, "1960-08-03")
        self.assertNoHolidayName(name_1960, range(1961, self.end_year))
        self.assertHolidayName(name_1961, (f"{year}-08-03" for year in range(1961, self.end_year)))
        self.assertNoHolidayName(name_1961, self.start_year)
        obs_dts = (
            "2008-08-04",
            "2014-08-04",
            "2025-08-04",
        )
        self.assertHolidayName(name_observed, obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)
        self.assertNoHolidayName(name_observed, range(self.start_year, 1998))

    def test_assumption_day(self):
        name = "Assomption"
        name_observed = f"{name} (observé)"
        self.assertNoHolidayName(name)
        self.assertOptionalHolidayName(name, (f"{year}-08-15" for year in self.full_range))
        obs_dts = (
            "2010-08-16",
            "2021-08-16",
        )
        self.assertOptionalHolidayName(name_observed, obs_dts)
        self.assertNoOptionalNonObservedHoliday(obs_dts)
        self.assertNoOptionalHolidayName(name_observed, range(self.start_year, 1998))

    def test_all_saints_day(self):
        name = "Toussaint"
        name_observed = f"{name} (observé)"
        self.assertNoHolidayName(name)
        self.assertOptionalHolidayName(name, (f"{year}-11-01" for year in self.full_range))
        obs_dts = (
            "2009-11-02",
            "2015-11-02",
            "2020-11-02",
        )
        self.assertOptionalHolidayName(name_observed, obs_dts)
        self.assertNoOptionalNonObservedHoliday(obs_dts)
        self.assertNoOptionalHolidayName(name_observed, range(self.start_year, 1998))

    def test_national_day(self):
        name = "Fête nationale"
        name_observed = f"{name} (observé)"
        self.assertHolidayName(name, (f"{year}-12-18" for year in self.full_range))
        obs_dts = (
            "2011-12-19",
            "2016-12-19",
            "2022-12-19",
        )
        self.assertHolidayName(name_observed, obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)
        self.assertNoHolidayName(name_observed, range(self.start_year, 1998))

    def test_christmas_day(self):
        name = "Noël"
        name_observed = f"{name} (observé)"
        self.assertHolidayName(name, (f"{year}-12-25" for year in self.full_range))
        obs_dts = (
            "2011-12-26",
            "2016-12-26",
            "2022-12-26",
        )
        self.assertHolidayName(name_observed, obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)
        self.assertNoHolidayName(name_observed, range(self.start_year, 1998))

    def test_islamic_new_year(self):
        name = "Jour de l'An musulman"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-08-21",
            "2021-08-10",
            "2022-07-30",
            "2023-07-19",
            "2024-07-06",
            "2025-06-27",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)
        obs_dts = (
            "2004-02-23",
            "2011-11-28",
        )
        self.assertIslamicNoEstimatedHolidayName(f"{name} (observé)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_prophets_birthday(self):
        name = "Mouloud"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-10-29",
            "2021-10-18",
            "2022-10-08",
            "2023-09-27",
            "2024-09-16",
            "2025-09-05",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)
        obs_dts = (
            "2004-05-03",
            "2012-02-06",
            "2019-11-11",
        )
        self.assertIslamicNoEstimatedHolidayName(f"{name} (observé)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_laylat_al_qadr(self):
        name = "Laylat al-Qadr"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-05-20",
            "2021-05-09",
            "2022-04-28",
            "2023-04-18",
            "2024-04-06",
            "2025-03-27",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)
        obs_dts = (
            "2000-12-25",
            "2008-09-29",
            "2013-08-05",
            "2021-05-10",
        )
        self.assertIslamicNoEstimatedHolidayName(f"{name} (observé)", obs_dts)

    def test_eid_al_fitr(self):
        name = "Korité"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-05-23",
            "2021-05-12",
            "2022-05-01",
            "2023-04-21",
            "2024-04-09",
            "2025-03-30",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)
        obs_dts = (
            "2004-11-15",
            "2012-08-20",
            "2022-05-02",
            "2025-03-31",
        )
        self.assertIslamicNoEstimatedHolidayName(f"{name} (observé)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_eid_al_adha(self):
        name = "Tabaski"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-07-31",
            "2021-07-20",
            "2022-07-10",
            "2023-06-28",
            "2024-06-16",
            "2025-06-07",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_day_after_eid_al_adha(self):
        name = "Lendemain de la Tabaski"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-08-01",
            "2021-07-21",
            "2022-07-11",
            "2023-06-29",
            "2024-06-17",
            "2025-06-08",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)
        obs_dts = (
            "2009-11-30",
            "2017-09-04",
            "2025-06-09",
        )
        self.assertIslamicNoEstimatedHolidayName(f"{name} (observé)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2025-01-01", "Jour de l'An"),
            ("2025-03-27", "Laylat al-Qadr"),
            ("2025-03-30", "Korité"),
            ("2025-03-31", "Korité (observé)"),
            ("2025-04-21", "Lundi de Pâques"),
            ("2025-04-24", "Fête nationale de la Concorde"),
            ("2025-05-01", "Journée internationale du travail"),
            ("2025-05-29", "Ascension"),
            ("2025-06-07", "Tabaski"),
            ("2025-06-08", "Lendemain de la Tabaski"),
            ("2025-06-09", "Lendemain de la Tabaski (observé); Lundi de Pentecôte"),
            ("2025-06-27", "Jour de l'An musulman"),
            ("2025-07-26", "Anniversaire du coup d'État du CNSP"),
            ("2025-08-03", "L'anniversaire de la proclamation de l'indépendance"),
            ("2025-08-04", "L'anniversaire de la proclamation de l'indépendance (observé)"),
            ("2025-08-15", "Assomption"),
            ("2025-09-05", "Mouloud"),
            ("2025-11-01", "Toussaint"),
            ("2025-12-18", "Fête nationale"),
            ("2025-12-25", "Noël"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2025-01-01", "New Year's Day"),
            ("2025-03-27", "Laylat al-Qadr"),
            ("2025-03-30", "Eid al-Fitr"),
            ("2025-03-31", "Eid al-Fitr (observed)"),
            ("2025-04-21", "Easter Monday"),
            ("2025-04-24", "National Concord Day"),
            ("2025-05-01", "International Labor Day"),
            ("2025-05-29", "Ascension"),
            ("2025-06-07", "Eid al-Adha"),
            ("2025-06-08", "Day after Eid al-Adha"),
            ("2025-06-09", "Day after Eid al-Adha (observed); Whit Monday"),
            ("2025-06-27", "Islamic New Year"),
            ("2025-07-26", "Anniversary of the CNSP Coup"),
            ("2025-08-03", "Anniversary of the Proclamation of Independence"),
            ("2025-08-04", "Anniversary of the Proclamation of Independence (observed)"),
            ("2025-08-15", "Assumption Day"),
            ("2025-09-05", "Prophet's Birthday"),
            ("2025-11-01", "All Saints' Day"),
            ("2025-12-18", "National Day"),
            ("2025-12-25", "Christmas Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

"""Generate SUMMARY.md for literate-nav with list of the countries and markets files."""

from pathlib import Path

import mkdocs_gen_files


def main():
    root = Path(__file__).parents[2]  # project folder
    holidays_path = root / "holidays"

    entities_data = [
        "* [Overview](index.md)",
        "* [Additional Examples](examples.md)",
        "* [Holiday Categories](holiday_categories.md)",
        "* [Changelog](changelog.md)",
        "* [Contributing](contributing.md)",
        "* [API Reference](api.md)",
        "* [Downloads](downloads.md)",
        "* Supported Entities",
    ]

    for entity_type in ("countries", "financial"):
        entities_data.append(f"    * holidays.{entity_type}")
        for path in sorted((holidays_path / entity_type).rglob("*.py")):
            file_name = path.stem
            if file_name == "__init__":
                continue
            entities_data.append(f"        * [{file_name}](auto_gen_docs/{file_name}.md)")

    with mkdocs_gen_files.open("SUMMARY.md", "w") as f:
        f.write("\n".join(entities_data))


if __name__ in {"__main__", "<run_path>"}:
    main()



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.tunisia import Tunisia
from tests.common import CommonCountryTests


class TestTunisia(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Tunisia)

    def test_2021(self):
        self.assertHolidayDatesInYear(
            2021,
            "2021-01-01",
            "2021-01-14",
            "2021-03-20",
            "2021-04-09",
            "2021-05-01",
            "2021-05-13",
            "2021-05-14",
            "2021-05-15",
            "2021-07-19",
            "2021-07-20",
            "2021-07-21",
            "2021-07-22",
            "2021-07-25",
            "2021-08-09",
            "2021-08-13",
            "2021-10-15",
            "2021-10-18",
        )

    def test_hijri_based(self):
        self.assertHoliday(
            # Eid al-Fitr
            "2021-05-13",
            "2021-05-14",
            "2021-05-15",
            # Eid al-Adha
            "2006-01-10",
            "2006-12-31",
            "2021-07-19",
            "2021-07-20",
            "2021-07-21",
            "2021-07-22",
            # Islamic New Year
            "2008-01-10",
            "2008-12-29",
            "2021-08-09",
            # Prophet Muhammad's Birthday
            "2021-10-18",
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2023-01-01", "رأس السنة الميلادية"),
            ("2023-01-14", "عيد الثورة والشباب"),
            ("2023-03-20", "عيد الإستقلال"),
            ("2023-04-09", "عيد الشهداء"),
            ("2023-04-21", "عيد الفطر (تقديري)"),
            ("2023-04-22", "عطلة عيد الفطر (تقديري)"),
            ("2023-04-23", "عطلة عيد الفطر (تقديري)"),
            ("2023-05-01", "عيد العمال"),
            ("2023-06-27", "يوم عرفة (تقديري)"),
            ("2023-06-28", "عيد الأضحى (تقديري)"),
            ("2023-06-29", "عطلة عيد الأضحى (تقديري)"),
            ("2023-06-30", "عطلة عيد الأضحى (تقديري)"),
            ("2023-07-19", "رأس السنة الهجرية (تقديري)"),
            ("2023-07-25", "عيد الجمهورية"),
            ("2023-08-13", "عيد المرأة"),
            ("2023-09-27", "عيد المولد النبوي (تقديري)"),
            ("2023-10-15", "عيد الجلاء"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2023-01-01", "New Year's Day"),
            ("2023-01-14", "Revolution and Youth Day"),
            ("2023-03-20", "Independence Day"),
            ("2023-04-09", "Martyrs' Day"),
            ("2023-04-21", "Eid al-Fitr (estimated)"),
            ("2023-04-22", "Eid al-Fitr Holiday (estimated)"),
            ("2023-04-23", "Eid al-Fitr Holiday (estimated)"),
            ("2023-05-01", "Labor Day"),
            ("2023-06-27", "Arafat Day (estimated)"),
            ("2023-06-28", "Eid al-Adha (estimated)"),
            ("2023-06-29", "Eid al-Adha Holiday (estimated)"),
            ("2023-06-30", "Eid al-Adha Holiday (estimated)"),
            ("2023-07-19", "Islamic New Year (estimated)"),
            ("2023-07-25", "Republic Day"),
            ("2023-08-13", "Women's Day"),
            ("2023-09-27", "Prophet's Birthday (estimated)"),
            ("2023-10-15", "Evacuation Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.constants import UNOFFICIAL
from holidays.countries.aland_islands import AlandIslands
from tests.common import CommonCountryTests


class TestAland(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(AlandIslands)

    def test_no_holidays(self):
        super().test_no_holidays()

        self.assertNoHolidays(
            AlandIslands(categories=UNOFFICIAL, years=range(self.start_year, 1950))
        )

    def test_alands_autonomy_day(self):
        name = "Ahvenanmaan itsehallintopäivä"
        self.assertHolidayName(name, (f"{year}-06-09" for year in range(1993, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1993))

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "Uudenvuodenpäivä"),
            ("2022-01-06", "Loppiainen"),
            ("2022-02-05", "Runebergin päivä"),
            ("2022-02-28", "Kalevalan päivä, suomalaisen kulttuurin päivä"),
            ("2022-03-19", "Minna Canthin päivä, tasa-arvon päivä"),
            ("2022-04-09", "Mikael Agricolan päivä, suomen kielen päivä"),
            ("2022-04-15", "Pitkäperjantai"),
            ("2022-04-17", "Pääsiäispäivä"),
            ("2022-04-18", "Toinen pääsiäispäivä"),
            ("2022-04-27", "Kansallinen veteraanipäivä"),
            ("2022-05-01", "Vappu"),
            ("2022-05-08", "Äitienpäivä"),
            ("2022-05-09", "Eurooppa-päivä"),
            ("2022-05-12", "J.V. Snellmanin päivä, suomalaisuuden päivä"),
            ("2022-05-15", "Kaatuneitten muistopäivä"),
            ("2022-05-26", "Helatorstai"),
            ("2022-06-05", "Helluntaipäivä"),
            ("2022-06-06", "Puolustusvoimain lippujuhlan päivä"),
            ("2022-06-09", "Ahvenanmaan itsehallintopäivä"),
            ("2022-06-24", "Juhannusaatto"),
            ("2022-06-25", "Juhannuspäivä; Suomen lipun päivä"),
            ("2022-07-06", "Eino Leinon päivä, runon ja suven päivä"),
            ("2022-10-01", "Miina Sillanpään ja kansalaisvaikuttamisen päivä"),
            ("2022-10-10", "Aleksis Kiven päivä, suomalaisen kirjallisuuden päivä"),
            ("2022-10-24", "YK:n päivä"),
            ("2022-11-05", "Pyhäinpäivä"),
            ("2022-11-06", "Ruotsalaisuuden päivä, Kustaa Aadolfin päivä"),
            ("2022-11-13", "Isänpäivä"),
            ("2022-11-20", "Lapsen oikeuksien päivä"),
            ("2022-12-06", "Itsenäisyyspäivä"),
            ("2022-12-08", "Jean Sibeliuksen päivä, suomalaisen musiikin päivä"),
            ("2022-12-24", "Jouluaatto"),
            ("2022-12-25", "Joulupäivä"),
            ("2022-12-26", "Tapaninpäivä"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-01-06", "Epiphany"),
            ("2022-02-05", "Runeberg Day"),
            ("2022-02-28", "Kalevala Day, Day of Finnish Culture"),
            ("2022-03-19", "Minna Canth Day, Day of Equality"),
            ("2022-04-09", "Mikael Agricola Day, Day of the Finnish Language"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-17", "Easter Sunday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-04-27", "National War Veterans' Day"),
            ("2022-05-01", "May Day"),
            ("2022-05-08", "Mother's Day"),
            ("2022-05-09", "Europe Day"),
            ("2022-05-12", "J. V. Snellman Day, Day of Finnish Heritage"),
            ("2022-05-15", "Remembrance Day"),
            ("2022-05-26", "Ascension Day"),
            ("2022-06-05", "Whit Sunday"),
            ("2022-06-06", "Flag Day of the Finnish Defense Forces"),
            ("2022-06-09", "Åland's Autonomy Day"),
            ("2022-06-24", "Midsummer Eve"),
            ("2022-06-25", "Day of the Finnish Flag; Midsummer Day"),
            ("2022-07-06", "Eino Leino Day, Day of Summer and Poetry"),
            ("2022-10-01", "Miina Sillanpää Day, Day of Civic Participation"),
            ("2022-10-10", "Aleksis Kivi Day, Day of Finnish Literature"),
            ("2022-10-24", "United Nations Day"),
            ("2022-11-05", "All Saints' Day"),
            ("2022-11-06", "Finnish Swedish Heritage Day, svenska dagen"),
            ("2022-11-13", "Father's Day"),
            ("2022-11-20", "Day of Children's Rights"),
            ("2022-12-06", "Independence Day"),
            ("2022-12-08", "Jean Sibelius Day, Day of Finnish Music"),
            ("2022-12-24", "Christmas Eve"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Second Day of Christmas"),
        )

    def test_l10n_sv_fi(self):
        self.assertLocalizedHolidays(
            "sv_FI",
            ("2022-01-01", "Nyårsdagen"),
            ("2022-01-06", "Trettondag"),
            ("2022-02-05", "Runebergsdagen"),
            ("2022-02-28", "Kalevaladagen, den finska kulturens dag"),
            ("2022-03-19", "Minna Canth-dagen, jämställdhetsdagen"),
            ("2022-04-09", "Mikael Agricoladagen, finska språkets dag"),
            ("2022-04-15", "Långfredagen"),
            ("2022-04-17", "Påskdagen"),
            ("2022-04-18", "Annandag påsk"),
            ("2022-04-27", "Nationella veterandagen"),
            ("2022-05-01", "Första maj"),
            ("2022-05-08", "Mors dag"),
            ("2022-05-09", "Europadagen"),
            ("2022-05-12", "Snellmansdagen, finskhetens dag"),
            ("2022-05-15", "De stupades dag"),
            ("2022-05-26", "Kristi himmelsfärdsdag"),
            ("2022-06-05", "Pingst"),
            ("2022-06-06", "Dagen för försvarets fanfest"),
            ("2022-06-09", "Ålands självstyrelsedag"),
            ("2022-06-24", "Midsommarafton"),
            ("2022-06-25", "Finlands flaggas dag; Midsommardagen"),
            ("2022-07-06", "Eino Leino-dagen, diktens och sommarens dag"),
            ("2022-10-01", "Miina Sillanpää-dagen, medborgarinflytandets dag"),
            ("2022-10-10", "Aleksis Kivi-dagen, den finska litteraturens dag"),
            ("2022-10-24", "FN-dagen"),
            ("2022-11-05", "Alla helgons dag"),
            ("2022-11-06", "Svenska dagen, Gustav Adolfsdagen"),
            ("2022-11-13", "Fars dag"),
            ("2022-11-20", "Barnkonventionens dag"),
            ("2022-12-06", "Självständighetsdagen"),
            ("2022-12-08", "Sibeliusdagen, den finländska musikens dag"),
            ("2022-12-24", "Julafton"),
            ("2022-12-25", "Juldagen"),
            ("2022-12-26", "Annandag jul"),
        )

    def test_l10n_th(self):
        self.assertLocalizedHolidays(
            "th",
            ("2022-01-01", "วันขึ้นปีใหม่"),
            ("2022-01-06", "วันสมโภชพระคริสต์แสดงองค์"),
            ("2022-02-05", "วันรูนแบร์ก"),
            ("2022-02-28", "วันกาเลวาลา, วันวัฒนธรรมฟินแลนด์"),
            ("2022-03-19", "วันมินน่า คานท์, วันแห่งความเสมอภาค"),
            ("2022-04-09", "วันมิคาเอล อากริโคลา, วันภาษาฟินแลนด์"),
            ("2022-04-15", "วันศุกร์ประเสริฐ"),
            ("2022-04-17", "วันอาทิตย์อีสเตอร์"),
            ("2022-04-18", "วันจันทร์อีสเตอร์"),
            ("2022-04-27", "วันทหารผ่านศึกแห่งชาติ"),
            ("2022-05-01", "วันเมย์เดย์ (วันแรงงาน)"),
            ("2022-05-08", "วันแม่"),
            ("2022-05-09", "วันยุโรป"),
            ("2022-05-12", "วันเจ.วี. สเนลล์มาน, วันมรดกฟินแลนด์"),
            ("2022-05-15", "วันรำลึก"),
            ("2022-05-26", "วันสมโภชพระเยซูเจ้าเสด็จขึ้นสวรรค์"),
            ("2022-06-05", "วันสมโภชพระจิตเจ้า"),
            ("2022-06-06", "วันกองกำลังป้องกันฟินแลนด์"),
            ("2022-06-09", "วันปกครองตนเองหมู่เกาะโอลันด์"),
            ("2022-06-24", "วันมิดซัมเมอร์อีฟ (วันก่อนวันกลางฤดูร้อน)"),
            ("2022-06-25", "วันธงชาติฟินแลนด์; วันมิดซัมเมอร์ (วันกลางฤดูร้อน)"),
            ("2022-07-06", "วันไอนอ ไลโน, วันแห่งบทกวีและฤดูร้อน"),
            ("2022-10-01", "วันมีนา ซิลลันแป, วันส่งเสริมการมีส่วนร่วมของพลเมือง"),
            ("2022-10-10", "วันอเล็กซิส กีวี, วันวรรณกรรมฟินแลนด์"),
            ("2022-10-24", "วันสหประชาชาติ"),
            ("2022-11-05", "วันสมโภชนักบุญทั้งหลาย"),
            ("2022-11-06", "วันมรดกสวีเดน-ฟินแลนด์, วันกุสตาฟวัส อดอลฟัส"),
            ("2022-11-13", "วันพ่อ"),
            ("2022-11-20", "วันสิทธิเด็ก"),
            ("2022-12-06", "วันประกาศอิสรภาพฟินแลนด์"),
            ("2022-12-08", "วันฌอง ซิเบลิอุส, วันดนตรีฟินแลนด์"),
            ("2022-12-24", "วันคริสต์มาสอีฟ"),
            ("2022-12-25", "วันคริสต์มาส"),
            ("2022-12-26", "วันคริสต์มาสวันที่สอง"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2022-01-01", "Новий рік"),
            ("2022-01-06", "Богоявлення"),
            ("2022-02-05", "День Рунеберга"),
            ("2022-02-28", "День Калевали, День фінської культури"),
            ("2022-03-19", "День Мінни Кант, День рівності"),
            ("2022-04-09", "День Мікаеля Аґріколи, День фінської мови"),
            ("2022-04-15", "Страсна пʼятниця"),
            ("2022-04-17", "Великдень"),
            ("2022-04-18", "Великодній понеділок"),
            ("2022-04-27", "Національний день ветеранів"),
            ("2022-05-01", "Ваппу"),
            ("2022-05-08", "День матері"),
            ("2022-05-09", "День Європи"),
            ("2022-05-12", "День Ю. В. Снелльмана, День фінської спадщини"),
            ("2022-05-15", "День ветеранів Національної війни"),
            ("2022-05-26", "Вознесіння Господнє"),
            ("2022-06-05", "Трійця"),
            ("2022-06-06", "День прапора фінських сил оборони"),
            ("2022-06-09", "День автономії Аландських островів"),
            ("2022-06-24", "Переддень літнього сонцестояння"),
            ("2022-06-25", "День літнього сонцестояння; День прапора Фінляндії"),
            ("2022-07-06", "День Ейно Лейно, День літа та поезії"),
            ("2022-10-01", "День Міїни Сілланпяя, День громадянської активності"),
            ("2022-10-10", "День Алексіса Ківі, День фінської літератури"),
            ("2022-10-24", "День ООН"),
            ("2022-11-05", "День усіх святих"),
            ("2022-11-06", "День фінської шведської спадщини, шведський день"),
            ("2022-11-13", "День батька"),
            ("2022-11-20", "День прав дитини"),
            ("2022-12-06", "День незалежності"),
            ("2022-12-08", "День Жана Сібеліуса, День фінської музики"),
            ("2022-12-24", "Святий вечір"),
            ("2022-12-25", "Різдво Христове"),
            ("2022-12-26", "Другий день Різдва"),
        )



#!/usr/bin/env python3

#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

import argparse
import json
import re
import sys
import time
from pathlib import Path

import requests
from requests.adapters import HTTPAdapter
from urllib3.util.retry import Retry

FIND_URL_PATTERN = re.compile(r'https?://[^\s<>"]+')
IGNORED_DIRECTORIES = {"__pycache__"}
IGNORE_DOMAINS = [
    r"archive\.is",
    r"archive\.org",
    r"archive\.ph",
    r"archive\.today",
    r"docs\.python\.org",
    r"github\.com",
    r"iso\.org",
    r"loc\.gov",
    r"namu\.wiki",
    r"semver\.org",
    r"web\.archive\.org",
    r"wikipedia\.org",
    r"wikisource\.org",
]
IGNORE_URL_REGEX = re.compile(r"|".join(IGNORE_DOMAINS))
# Characters that can be part of a URL - used to avoid replacing URLs that are part of larger URLs.
CDX_API_URL = "https://web.archive.org/cdx/search/cdx"
SAVE_API_URL = "https://web.archive.org/save"
REQUEST_TIMEOUT = 60
USER_AGENT = "https://github.com/vacanza/holidays (python)"


class WMArchiver:
    def __init__(self) -> None:
        self.args: argparse.Namespace = self.get_args()
        self.session: requests.Session = self.get_session()
        self.url_map: dict[str, str | None] = {}

    @staticmethod
    def get_args() -> argparse.Namespace:
        parser = argparse.ArgumentParser(
            description="Find, check/archive via Wayback Machine, and replace URLs "
            "in project files.",
            formatter_class=argparse.ArgumentDefaultsHelpFormatter,
        )
        parser.add_argument(
            "path",
            nargs="?",
            help="Optional path to a specific file or directory to scan. "
            "If omitted, scans the entire 'holidays' library.",
        )
        parser.add_argument(
            "--archive-policy",
            choices=["if-missing", "always", "never"],
            default="if-missing",
            help="Archiving policy: 'if-missing' (only archive if no capture exists), "
            "'always' (always submit for archiving), 'never' (only search for "
            "existing captures)",
        )
        mode_group = parser.add_mutually_exclusive_group()
        mode_group.add_argument(
            "--show-only",
            action="store_true",
            help="Only print URLs that would need to be archived without checking or modifying "
            "anything. Exits with code 1 if any URLs are found.",
        )
        mode_group.add_argument(
            "--check-liveness",
            action="store_true",
            help="Check if original URLs are still alive by sending HTTP HEAD requests. "
            "Prints dead links and exits with code 1 if any are found.",
        )
        return parser.parse_args()

    @staticmethod
    def get_session() -> requests.Session:
        session = requests.Session()
        retries = Retry(
            total=3,
            backoff_factor=1,
            status_forcelist=[429, 500, 502, 503, 504],
            allowed_methods=["HEAD", "GET"],
        )
        adapter = HTTPAdapter(max_retries=retries)
        session.mount("http://", adapter)  # NOSONAR
        session.mount("https://", adapter)
        session.headers = {"User-Agent": USER_AGENT}
        return session

    @staticmethod
    def find_links_in_file(filepath: Path) -> list[str]:
        """Finds all hyperlinks within a single text file, respecting IGNORE_DOMAINS."""
        try:
            file_content = filepath.read_text(encoding="utf-8")
        except Exception as e:
            print(f"Warning: Could not read file {filepath} due to: {e}", file=sys.stderr)
            return []

        found_urls = {
            match.rstrip(")")
            for match in FIND_URL_PATTERN.findall(file_content)
            if match and not IGNORE_URL_REGEX.search(match)
        }
        return sorted(found_urls)

    def scan_directory_for_links(self, directory: Path) -> dict[Path, list[str]]:
        """Walks a directory and finds hyperlinks in .py files.

        Returns:
            A dictionary mapping {filepath: [urls]}
        """
        if not directory.is_dir():
            print(
                f"Error: Specified path '{directory}' is not a valid directory.", file=sys.stderr
            )
            return {}

        print(f"Scanning directory: {directory}")
        file_to_urls_map: dict[Path, list[str]] = {}
        all_unique_urls: set[str] = set()
        for path in directory.rglob("*.py"):
            if not path.is_file() or any(part in IGNORED_DIRECTORIES for part in path.parts):
                continue

            print(f"  Processing: {path}")
            if urls_in_file := self.find_links_in_file(path):
                file_to_urls_map[path] = urls_in_file
                all_unique_urls.update(urls_in_file)

        print(
            f"\nScan complete. Found links in {len(file_to_urls_map)} file(s). "
            f"Found {len(all_unique_urls)} unique URLs."
        )
        return file_to_urls_map

    def check_availability_api(self, url: str) -> str | None:
        """Checks the Wayback Machine CDX API for the latest available capture.

        Returns:
            The Wayback URL of the latest capture, or None if not found or error.
        """
        try:
            params: dict[str, str | int] = {
                "url": url,
                "output": "json",
                "limit": -1,
                "fastLatest": "true",
                "filter": "statuscode:200",
            }
            response = self.session.get(CDX_API_URL, params=params, timeout=REQUEST_TIMEOUT)
            response.raise_for_status()
            data = response.json()
            if len(data) > 1:
                # The first row is the header, the rest are captures
                latest = data[1]
                timestamp = latest[1]
                original = latest[2]
                wayback_url = f"https://web.archive.org/web/{timestamp}/{original}"
                return wayback_url
        except requests.exceptions.Timeout:
            print(f"\tError: CDX API request timed out for {url}", file=sys.stderr)
        except requests.exceptions.RequestException as e:
            status_code = getattr(e.response, "status_code", "N/A")
            if isinstance(e, json.JSONDecodeError):
                print(
                    f"\tError: CDX API returned non-JSON response for {url}. "
                    f"Status: {status_code}",
                    file=sys.stderr,
                )
            else:
                print(
                    f"\tError checking CDX API for {url} (Status: {status_code}): {e}",
                    file=sys.stderr,
                )
        except Exception as e:
            print(f"\tUnexpected error during CDX API check for {url}: {e}", file=sys.stderr)
        return None

    def archive_url(self, url: str) -> None:
        """Submits a URL to the Wayback Machine save API using POST.

        Returns:
            Nothing; success is determined by subsequent availability check.
        """
        print(f"\tSubmitting for archive: {url}")
        try:
            response = self.session.post(
                SAVE_API_URL, data={"url": url}, timeout=REQUEST_TIMEOUT, allow_redirects=True
            )
            response.raise_for_status()
            print(f"\tArchive submission complete (status: {response.status_code})")
        except requests.exceptions.Timeout:
            print(f"\tError: Archive request timed out for {url}", file=sys.stderr)
        except requests.exceptions.RequestException as e:
            print(f"\tError archiving {url}: {e}", file=sys.stderr)

    @staticmethod
    def replace_urls_in_file(
        filepath: Path, file_urls: list[str], url_map: dict[str, str]
    ) -> bool:
        """Replaces original URLs with their Wayback Machine counterparts in a file.

        Uses the provided valid url_map {original_url: wayback_url}.
        Only replaces URLs specifically listed in file_urls for this path.

        Returns:
            True if replacements were made, False otherwise.
        """
        try:
            file_content = filepath.read_text(encoding="utf-8")
        except Exception as e:
            print(f"Error reading file for replacement: {filepath} - {e}", file=sys.stderr)
            return False

        modified_content = file_content
        replacements_made = 0

        valid_replacements_for_file = [
            (url, mapped_url) for url in file_urls if (mapped_url := url_map.get(url))
        ]
        if not valid_replacements_for_file:
            return False

        print(f"  Attempting replacements in: {filepath}")
        for original_url, wayback_url in valid_replacements_for_file:
            escaped_original_url = re.escape(original_url)
            # pattern that matches the URL when it's not part of a larger URL;
            # negative lookbehind: ensure URL doesn't follow a character that could be
            # part of a URL;
            # negative lookahead: ensure URL isn't followed by a character that could be
            # part of a URL.
            pattern = rf'(?<![^\s<>()"]){escaped_original_url}(?![^\s<>()"])'
            new_content, count = re.subn(pattern, wayback_url, modified_content)
            if count > 0:
                modified_content = new_content
                replacements_made += count
                print(f"    Replaced '{original_url}' ({count}x)")

        if replacements_made > 0:
            try:
                filepath.write_text(modified_content, encoding="utf-8", newline="\n")
                print(f"  Successfully wrote {replacements_made} replacement(s) to {filepath}")
                return True
            except Exception as e:
                print(f"Error writing modified file: {filepath} - {e}", file=sys.stderr)
                return False

        return False

    def check_archive_urls_in_file(self, urls: list[str]) -> dict[str, str]:
        archive_policy = self.args.archive_policy
        file_url_map: dict[str, str] = {}

        for url in urls:
            if not (wayback_url := self.url_map.get(url)):
                if existing_capture := self.check_availability_api(url):
                    if archive_policy in {"if-missing", "never"}:
                        print(
                            f"\tUsing existing capture for {url} "
                            f"(archive_policy={archive_policy})."
                        )
                        wayback_url = existing_capture
                    else:
                        print(
                            f"\tIgnoring existing capture for {url} (archive_policy=always), "
                            f"will attempt archive."
                        )
                elif archive_policy == "never":
                    print(
                        f"\tExisting capture not found for {url}, not archiving "
                        f"(archive_policy=never)."
                    )
                    continue

                if not wayback_url:
                    self.archive_url(url)
                    wayback_url = self.check_availability_api(url)

            if wayback_url:
                file_url_map[url] = wayback_url
                self.url_map[url] = wayback_url

        return file_url_map

    @staticmethod
    def run_show_only_mode(files_to_urls_data: dict[Path, list[str]]) -> int:
        """Runs the show-only mode which just displays URLs without checking or modifying.

        Args:
            files_to_urls_data: Dictionary mapping files to their URLs

        Returns:
            Exit code (0 for success, 1 if URLs are found)
        """
        print("Check-only mode: Printing all URLs that would need archiving")

        if any(files_to_urls_data.values()):
            print("FOUND: URLs that need archiving")
            for source_file, urls in files_to_urls_data.items():
                print(f"\nFile: {source_file}")
                for url in urls:
                    print(f"  {url}")
            exit_code = 1
        else:
            print("SUCCESS: No URLs to archive")
            exit_code = 0

        return exit_code

    def run_check_liveness_mode(self, files_to_urls_data: dict[Path, list[str]]) -> int:
        """Checks if URLs are alive by sending HTTP HEAD requests.

        Args:
            files_to_urls_data: Dictionary mapping files to their URLs

        Returns:
            Exit code (0 if all links alive, 1 if any dead links found)
        """
        print("Liveness check mode: Checking if URLs are still alive")
        total_urls = 0
        dead_count = 0

        for source_file, urls_in_file in files_to_urls_data.items():
            print(f"Checking file: {source_file}")

            for url in urls_in_file:
                total_urls += 1
                try:
                    # use shorter timeout for HEAD requests.
                    response = self.session.head(
                        url, timeout=REQUEST_TIMEOUT // 4, allow_redirects=True
                    )
                    if response.status_code in {405, 501}:
                        response = self.session.get(
                            url, timeout=REQUEST_TIMEOUT // 4, allow_redirects=True
                        )
                    response.raise_for_status()
                except requests.exceptions.RequestException as e:
                    print(f"  DEAD: {url} - {e}")
                    dead_count += 1

        print("\n--- Liveness Check Summary ---")
        print(f"Total URLs checked: {total_urls}")
        print(f"Dead links found: {dead_count}")
        return 1 if dead_count > 0 else 0

    def run_archive_mode(self, files_to_urls_data: dict[Path, list[str]]) -> int:
        """Archives URLs via Wayback Machine and replaces them in the files.

        Args:
            files_to_urls_data: Dictionary mapping files to their URLs

        Returns:
            Exit code (always 0 for success)
        """
        if not files_to_urls_data:
            print("No URLs found or no files processed. Exiting.")
            return 0

        print("\n--- Stage: Per-File Processing ---")
        total_files_processed = 0
        total_files_modified = 0

        for source_file, urls in files_to_urls_data.items():
            print(f"\nProcessing file: {source_file}")
            file_url_map = self.check_archive_urls_in_file(urls)
            total_files_processed += 1
            if self.replace_urls_in_file(source_file, urls, file_url_map):
                total_files_modified += 1

        print(f"Processed {total_files_processed} files. Modified {total_files_modified} files.")
        return 1 if total_files_modified != total_files_processed else 0

    def run(self) -> None:
        start_time_total = time.time()
        target_path = (
            Path(self.args.path).absolute()
            if self.args.path
            else Path(__file__).parents[1] / "holidays"
        )
        files_to_urls_data: dict[Path, list[str]] = {}
        if target_path.is_file():
            files_to_urls_data = {target_path: self.find_links_in_file(target_path)}
        elif target_path.is_dir():
            files_to_urls_data = self.scan_directory_for_links(target_path)
        else:
            print(f"Error: Path not found: {target_path}", file=sys.stderr)
            sys.exit(1)

        if self.args.show_only:
            exit_code = self.run_show_only_mode(files_to_urls_data)
        elif self.args.check_liveness:
            exit_code = self.run_check_liveness_mode(files_to_urls_data)
        else:
            exit_code = self.run_archive_mode(files_to_urls_data)

        end_time_total = time.time()
        print("-" * 30)
        print(f"Script finished in {end_time_total - start_time_total:.2f} seconds.")
        print("Done.")

        sys.exit(exit_code)


if __name__ == "__main__":
    WMArchiver().run()



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.constants import HALF_DAY
from holidays.countries.estonia import Estonia
from tests.common import CommonCountryTests


class TestEstonia(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Estonia)

    def test_no_holidays(self):
        super().test_no_holidays()

        self.assertNoHolidays(Estonia(categories=HALF_DAY, years=range(self.start_year, 2001)))

    def test_new_years_day(self):
        self.assertHolidayName("uusaasta", (f"{year}-01-01" for year in self.full_range))

    def test_independence_day(self):
        self.assertHolidayName("iseseisvuspäev", (f"{year}-02-24" for year in self.full_range))

    def test_good_friday(self):
        name = "suur reede"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, range(1994, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 1994))

    def test_easter_sunday(self):
        name = "ülestõusmispühade 1. püha"
        self.assertHolidayName(
            name,
            "2020-04-12",
            "2021-04-04",
            "2022-04-17",
            "2023-04-09",
            "2024-03-31",
            "2025-04-20",
        )
        self.assertHolidayName(name, range(1994, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 1994))

    def test_may_day(self):
        name_1991 = "töörahva rahvusvahelise solidaarsuse päev"
        name_1994 = "kevadpüha"
        self.assertHolidayName(
            name_1991, (f"{year}-05-01" for year in range(self.start_year, 1994))
        )
        self.assertHolidayName(name_1994, (f"{year}-05-01" for year in range(1994, self.end_year)))
        self.assertNoHolidayName(name_1991, range(1994, self.end_year))
        self.assertNoHolidayName(name_1994, range(self.start_year, 1994))

    def test_whit_sunday(self):
        name = "nelipühade 1. püha"
        self.assertHolidayName(
            name,
            "2020-05-31",
            "2021-05-23",
            "2022-06-05",
            "2023-05-28",
            "2024-05-19",
            "2025-06-08",
        )
        self.assertHolidayName(name, range(1994, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 1994))

    def test_victory_day(self):
        self.assertHolidayName("võidupüha", (f"{year}-06-23" for year in self.full_range))

    def test_midsummer_day(self):
        self.assertHolidayName("jaanipäev", (f"{year}-06-24" for year in self.full_range))

    def test_restoration_of_independence_day(self):
        name = "taasiseseisvumispäev"
        self.assertHolidayName(name, (f"{year}-08-20" for year in range(1998, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1998))

    def test_day_of_declaration_of_sovereignty(self):
        name = "taassünnipäev"
        self.assertHolidayName(name, (f"{year}-11-16" for year in range(self.start_year, 1994)))
        self.assertNoHolidayName(name, range(1994, self.end_year))

    def test_christmas_eve(self):
        name = "jõululaupäev"
        self.assertHolidayName(name, (f"{year}-12-24" for year in range(2005, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2005))

    def test_christmas_day(self):
        self.assertHolidayName("esimene jõulupüha", (f"{year}-12-25" for year in self.full_range))

    def test_second_christmas_day(self):
        self.assertHolidayName("teine jõulupüha", (f"{year}-12-26" for year in self.full_range))

    def test_pre_holiday_day(self):
        name = "pühade-eelne päev (tööpäev lüheneb 3 tunni võrra)"
        self.assertNoHolidayName(name)
        self.assertHalfDayHolidayName(
            name,
            (f"{year}-02-23" for year in range(2001, self.end_year)),
            (f"{year}-06-22" for year in range(2001, self.end_year)),
            (f"{year}-12-24" for year in range(2001, 2005)),
            (f"{year}-12-23" for year in range(2005, self.end_year)),
            (f"{year}-12-31" for year in range(2001, self.end_year)),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "uusaasta"),
            ("2022-02-23", "pühade-eelne päev (tööpäev lüheneb 3 tunni võrra)"),
            ("2022-02-24", "iseseisvuspäev"),
            ("2022-04-15", "suur reede"),
            ("2022-04-17", "ülestõusmispühade 1. püha"),
            ("2022-05-01", "kevadpüha"),
            ("2022-06-05", "nelipühade 1. püha"),
            ("2022-06-22", "pühade-eelne päev (tööpäev lüheneb 3 tunni võrra)"),
            ("2022-06-23", "võidupüha"),
            ("2022-06-24", "jaanipäev"),
            ("2022-08-20", "taasiseseisvumispäev"),
            ("2022-12-23", "pühade-eelne päev (tööpäev lüheneb 3 tunni võrra)"),
            ("2022-12-24", "jõululaupäev"),
            ("2022-12-25", "esimene jõulupüha"),
            ("2022-12-26", "teine jõulupüha"),
            ("2022-12-31", "pühade-eelne päev (tööpäev lüheneb 3 tunni võrra)"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-02-23", "Pre-holiday day (workday shortened by 3 hours)"),
            ("2022-02-24", "Independence Day"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-17", "Easter Sunday"),
            ("2022-05-01", "May Day"),
            ("2022-06-05", "Whit Sunday"),
            ("2022-06-22", "Pre-holiday day (workday shortened by 3 hours)"),
            ("2022-06-23", "Victory Day"),
            ("2022-06-24", "Midsummer Day"),
            ("2022-08-20", "Independence Restoration Day"),
            ("2022-12-23", "Pre-holiday day (workday shortened by 3 hours)"),
            ("2022-12-24", "Christmas Eve"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Second Day of Christmas"),
            ("2022-12-31", "Pre-holiday day (workday shortened by 3 hours)"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2022-01-01", "Новий рік"),
            ("2022-02-23", "Передсвятковий робочий день (скорочений на 3 години)"),
            ("2022-02-24", "День незалежності"),
            ("2022-04-15", "Страсна пʼятниця"),
            ("2022-04-17", "Великдень"),
            ("2022-05-01", "День весни"),
            ("2022-06-05", "Трійця"),
            ("2022-06-22", "Передсвятковий робочий день (скорочений на 3 години)"),
            ("2022-06-23", "День Перемоги"),
            ("2022-06-24", "День літнього сонцестояння"),
            ("2022-08-20", "День відновлення незалежності"),
            ("2022-12-23", "Передсвятковий робочий день (скорочений на 3 години)"),
            ("2022-12-24", "Святий вечір"),
            ("2022-12-25", "Різдво Христове"),
            ("2022-12-26", "Другий день Різдва"),
            ("2022-12-31", "Передсвятковий робочий день (скорочений на 3 години)"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.armenia import Armenia
from tests.common import CommonCountryTests, WorkingDayTests


class TestArmenia(CommonCountryTests, WorkingDayTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Armenia)

    def test_new_years_day(self):
        name = "Նոր տարվա օր"
        self.assertHolidayName(
            name,
            (f"{year}-01-01" for year in self.full_range),
            (f"{year}-01-02" for year in self.full_range),
        )

    def test_christmas_and_epiphany_day(self):
        self.assertHolidayName(
            "Սուրբ Ծնունդ եւ Հայտնություն", (f"{year}-01-06" for year in self.full_range)
        )

    def test_christmas_holidays(self):
        name = "նախածննդյան տոներ"
        self.assertHolidayName(
            name,
            (f"{year}-01-03" for year in range(2010, 2022)),
            (f"{year}-01-04" for year in range(2010, 2022)),
            (f"{year}-01-05" for year in range(2010, 2022)),
        )
        self.assertNoHolidayName(name, range(self.start_year, 2010), range(2022, self.end_year))

    def test_day_of_remembrance_of_the_dead(self):
        name = "Մեռելոց հիշատակի օր"
        self.assertHolidayName(name, (f"{year}-01-07" for year in range(2010, 2022)))
        self.assertNoHolidayName(name, range(self.start_year, 2010), range(2022, self.end_year))

    def test_remembrance_for_fallen_defenders(self):
        name = "Հայրենիքի պաշտպանության համար զոհվածների հիշատակի օր"
        self.assertHolidayName(name, (f"{year}-01-27" for year in range(2026, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2026))

    def test_army_day(self):
        name = "Բանակի օր"
        self.assertHolidayName(name, (f"{year}-01-28" for year in range(2003, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2003))

        self.assertWorkdayHolidayName(name, "2002-01-28")
        self.assertNoWorkdayHolidayName(
            name, range(self.start_year, 2002), range(2003, self.end_year)
        )

    def test_womens_day(self):
        name_2001 = "Կանանց միջազգային օր"
        name_2002 = "Կանանց տոն"
        self.assertHolidayName(name_2001, "2001-03-08")
        self.assertHolidayName(name_2002, (f"{year}-03-08" for year in range(2002, self.end_year)))
        self.assertNoHolidayName(
            name_2001, range(self.start_year, 2001), range(2002, self.end_year)
        )
        self.assertNoHolidayName(name_2002, range(self.start_year, 2002))

    def test_motherhood_and_beauty_day(self):
        name_1994_2003 = "Մայրության և գեղեցկության տոն"
        name_2002 = "Մայրության, գեղեցկության եւ սիրո տոն"
        self.assertHolidayName(name_1994_2003, (f"{year}-04-07" for year in range(1994, 2002)))
        self.assertNoHolidayName(
            name_1994_2003, range(self.start_year, 1994), range(2002, self.end_year)
        )
        self.assertNoHolidayName(name_2002)

        self.assertWorkdayHolidayName(name_2002, "2002-04-07")
        self.assertWorkdayHolidayName(
            name_1994_2003, (f"{year}-04-07" for year in range(2003, self.end_year))
        )
        self.assertNoWorkdayHolidayName(
            name_2002, range(self.start_year, 2002), range(2003, self.end_year)
        )
        self.assertNoWorkdayHolidayName(name_1994_2003, range(self.start_year, 2003))

    def test_armenian_genocide_remembrance_day(self):
        name_1992 = "Ցեղասպանության զոհերի հիշատակի օր"
        name_2015 = "Հայոց ցեղասպանության զոհերի հիշատակի օր"
        self.assertHolidayName(
            name_1992, (f"{year}-04-24" for year in range(self.start_year, 2015))
        )
        self.assertHolidayName(name_2015, (f"{year}-04-24" for year in range(2015, self.end_year)))
        self.assertNoHolidayName(name_1992, range(2015, self.end_year))
        self.assertNoHolidayName(name_2015, range(self.start_year, 2015))

    def test_labor_day(self):
        name_2001 = "Աշխատավորների համերաշխության միջազգային օր"
        name_2002 = "Աշխատանքի օր"
        self.assertHolidayName(name_2001, "2001-05-01")
        self.assertHolidayName(name_2002, (f"{year}-05-01" for year in range(2002, self.end_year)))
        self.assertNoHolidayName(
            name_2001, range(self.start_year, 2001), range(2002, self.end_year)
        )
        self.assertNoHolidayName(name_2002, range(self.start_year, 2002))

    def test_victory_and_peace_day(self):
        name = "Հաղթանակի և Խաղաղության տոն"
        self.assertHolidayName(name, (f"{year}-05-09" for year in range(1995, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1995))

    def test_republic_day(self):
        self.assertHolidayName("Հանրապետության օր", (f"{year}-05-28" for year in self.full_range))

    def test_constitution_day(self):
        name = "Սահմանադրության օր"
        self.assertHolidayName(name, (f"{year}-07-05" for year in range(1996, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1996))

    def test_independence_day(self):
        name = "Անկախության օր"
        self.assertHolidayName(name, (f"{year}-09-21" for year in range(1992, self.end_year)))
        self.assertNoHolidayName(name, 1991)

    def test_new_years_eve(self):
        self.assertHolidayName("Նոր տարվա գիշեր", (f"{year}-12-31" for year in self.full_range))

    def test_mother_language_day(self):
        name = "Մայրենի լեզվի օր"
        self.assertNoHolidayName(name)

        self.assertWorkdayHolidayName(
            name, (f"{year}-02-21" for year in range(2006, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 2006))

    def test_saint_vardanants_day(self):
        name = "Սուրբ Վարդանանց տոն"
        self.assertNoHolidayName(name)

        self.assertWorkdayHolidayName(
            name,
            "2020-02-27",
            "2021-03-11",
            "2022-03-03",
            "2023-02-23",
            "2024-03-14",
            "2025-02-27",
        )
        self.assertWorkdayHolidayName(name, range(2002, self.end_year))
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 2002))

    def test_remembrance_of_victims_of_massacres_day(self):
        name = (
            "Ադրբեջանական ԽՍՀ-ում կազմակերպված ջարդերի զոհերի հիշատակի եւ "
            "բռնագաղթված հայ բնակչության իրավունքների պաշտպանության օր"
        )
        self.assertNoHolidayName(name)

        self.assertWorkdayHolidayName(
            name, (f"{year}-02-28" for year in range(2006, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 2006))

    def test_armenian_cinema_day(self):
        name = "Հայ կինոյի օր"
        self.assertNoHolidayName(name)

        self.assertWorkdayHolidayName(
            name, (f"{year}-04-16" for year in range(2020, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 2020))

    def test_taxpayer_day(self):
        name = "Հարկ վճարողի օր"
        self.assertNoHolidayName(name)

        self.assertWorkdayHolidayName(
            name, (f"{year}-04-18" for year in range(2022, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 2022))

    def test_citizen_of_republic_of_armenia_day(self):
        name = "Հայաստանի Հանրապետության քաղաքացու օր"
        self.assertNoHolidayName(name)

        self.assertWorkdayHolidayName(
            name,
            "2020-04-25",
            "2021-04-25",
            "2022-04-30",
            "2023-04-29",
            "2024-04-27",
            "2025-04-26",
        )
        self.assertWorkdayHolidayName(name, range(2019, self.end_year))
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 2019))

    def test_defenders_day(self):
        name = "Երկրապահի օր"
        self.assertNoHolidayName(name)

        self.assertWorkdayHolidayName(
            name, (f"{year}-05-08" for year in range(2002, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 2002))

    def test_family_day(self):
        name = "Ընտանիքի օր"
        self.assertNoHolidayName(name)

        self.assertWorkdayHolidayName(
            name, (f"{year}-05-15" for year in range(2012, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 2012))

    def test_students_and_youth_day(self):
        name = "Ուսանողների եւ երիտասարդների օր"
        self.assertNoHolidayName(name)

        self.assertWorkdayHolidayName(
            name, (f"{year}-05-16" for year in range(2016, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 2016))

    def test_childrens_rights_protection_day(self):
        name = "Երեխաների իրավունքների պաշտպանության օր"
        self.assertNoHolidayName(name)

        self.assertWorkdayHolidayName(
            name, (f"{year}-06-01" for year in range(2002, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 2002))

    def test_remembrance_of_repressed_day(self):
        name = "Բռնադատվածների հիշատակի օր"
        self.assertNoHolidayName(name)

        self.assertWorkdayHolidayName(
            name, (f"{year}-06-14" for year in range(2007, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 2007))

    def test_fathers_day(self):
        name = "Հայրերի օր"
        self.assertNoHolidayName(name)

        self.assertWorkdayHolidayName(
            name, (f"{year}-06-17" for year in range(2025, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 2025))

    def test_feast_of_holy_etchmiadzin(self):
        name = "Սուրբ Էջմիածնի տոն"
        self.assertNoHolidayName(name)

        self.assertWorkdayHolidayName(
            name,
            "2020-06-21",
            "2021-07-04",
            "2022-06-26",
            "2023-06-18",
            "2024-07-07",
            "2025-06-22",
        )
        self.assertWorkdayHolidayName(name, range(2002, self.end_year))
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 2002))

    def test_state_symbols_day(self):
        name = "Պետական խորհրդանիշների օր"
        self.assertNoHolidayName(name)

        self.assertWorkdayHolidayName(
            name, (f"{year}-07-05" for year in range(2017, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 2017))

    def test_remembrance_of_victims_of_sinjar_yazidi_genocide_day(self):
        name = "2014 թվականի՝ Սինջարի եզդիների ցեղասպանության զոհերի հիշատակի օր"
        self.assertNoHolidayName(name)

        self.assertWorkdayHolidayName(
            name, (f"{year}-08-03" for year in range(2024, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 2024))

    def test_knowledge_and_literacy_day(self):
        name_2001 = "Գիտելիքի, գրի եւ դպրության օր"
        name_2005 = "Գիտելիքի եւ դպրության օր"
        self.assertNoHolidayName(name_2001)
        self.assertNoHolidayName(name_2005)

        self.assertWorkdayHolidayName(name_2001, (f"{year}-09-01" for year in range(2001, 2005)))
        self.assertWorkdayHolidayName(
            name_2005, (f"{year}-09-01" for year in range(2005, self.end_year))
        )
        self.assertNoWorkdayHolidayName(
            name_2001, range(self.start_year, 2001), range(2005, self.end_year)
        )
        self.assertNoWorkdayHolidayName(name_2005, range(self.start_year, 2005))

    def test_national_minorities_day(self):
        name = "Հայաստանի Հանրապետության ազգային փոքրամասնությունների օր"
        self.assertNoHolidayName(name)

        self.assertWorkdayHolidayName(
            name,
            "2021-10-03",
            "2022-10-02",
            "2023-10-01",
            "2024-10-06",
            "2025-10-05",
        )
        self.assertWorkdayHolidayName(name, range(2021, self.end_year))
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 2021))

    def test_teachers_day(self):
        name = "Ուսուցչի օր"
        self.assertNoHolidayName(name)

        self.assertWorkdayHolidayName(
            name, (f"{year}-10-05" for year in range(2011, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 2011))

    def test_translators_day(self):
        name = "Թարգմանչաց տոն"
        self.assertNoHolidayName(name)

        self.assertWorkdayHolidayName(
            name,
            "2020-10-10",
            "2021-10-09",
            "2022-10-08",
            "2023-10-14",
            "2024-10-12",
            "2025-10-11",
        )
        self.assertWorkdayHolidayName(name, range(2001, self.end_year))
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 2001))

    def test_local_self_government_day(self):
        name = "Տեղական ինքնակառավարման օր"
        self.assertNoHolidayName(name)

        self.assertWorkdayHolidayName(
            name, (f"{year}-11-10" for year in range(2011, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 2011))

    def test_earthquake_victims_remembrance_day(self):
        name_2001 = "Երկրաշարժի զոհերի հիշատակի օր"
        name_2020 = "Երկրաշարժի զոհերի հիշատակի եւ աղետներին դիմակայունության օր"
        self.assertNoHolidayName(name_2001)
        self.assertNoHolidayName(name_2020)

        self.assertWorkdayHolidayName(name_2001, (f"{year}-12-07" for year in range(2001, 2020)))
        self.assertWorkdayHolidayName(
            name_2020, (f"{year}-12-07" for year in range(2020, self.end_year))
        )
        self.assertNoWorkdayHolidayName(
            name_2001, range(self.start_year, 2001), range(2020, self.end_year)
        )
        self.assertNoWorkdayHolidayName(name_2020, range(self.start_year, 2020))

    def test_condemnation_and_prevention_of_genocides_day(self):
        name = "Ցեղասպանությունների դատապարտման եւ կանխարգելման օր"
        self.assertNoHolidayName(name)

        self.assertWorkdayHolidayName(
            name, (f"{year}-12-09" for year in range(2015, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 2015))

    def test_substituted_holidays(self):
        self.assertHoliday(
            "1998-12-07",
            "2000-05-08",
            "2003-01-03",
            "2005-01-03",
            "2005-03-07",
            "2005-07-04",
            "2006-01-03",
            "2006-05-08",
            "2007-01-03",
            "2007-01-04",
            "2007-04-30",
            "2008-01-03",
            "2008-01-04",
            "2008-05-02",
            "2008-06-30",
            "2008-08-18",
            "2008-09-15",
            "2009-01-05",
            "2009-01-07",
            "2009-04-13",
            "2009-07-20",
            "2009-08-17",
            "2009-09-14",
            "2010-01-08",
            "2010-04-05",
            "2010-07-12",
            "2010-08-16",
            "2010-09-13",
            "2010-09-20",
            "2011-03-07",
            "2011-04-25",
            "2011-07-04",
            "2011-08-01",
            "2011-08-15",
            "2011-09-12",
            "2012-03-09",
            "2012-04-09",
            "2012-04-30",
            "2012-07-06",
            "2012-07-16",
            "2012-08-13",
            "2012-09-17",
            "2013-04-01",
            "2013-05-10",
            "2013-05-27",
            "2013-07-08",
            "2013-08-19",
            "2013-09-16",
            "2013-12-30",
            "2014-01-27",
            "2014-04-21",
            "2014-05-02",
            "2014-07-28",
            "2014-08-18",
            "2014-09-15",
            "2015-01-08",
            "2015-01-09",
            "2015-04-06",
            "2015-04-23",
            "2015-07-13",
            "2015-09-14",
            "2016-03-07",
            "2016-03-28",
            "2016-09-12",
            "2017-04-17",
            "2017-05-08",
            "2017-09-18",
            "2018-03-09",
            "2018-04-02",
            "2018-04-30",
            "2018-10-11",
            "2018-10-12",
            "2020-01-27",
            "2020-05-29",
            "2021-09-20",
        )
        self.assertSubdivLoHoliday("2018-12-07")
        self.assertSubdivShHoliday("2018-12-07")

    def test_workdays(self):
        self.assertWorkingDay(
            "1998-12-12",
            "2000-05-06",
            "2002-12-29",
            "2005-01-08",
            "2005-03-05",
            "2005-07-02",
            "2006-01-08",
            "2006-05-06",
            "2006-12-30",
            "2007-01-07",
            "2007-04-28",
            "2007-12-29",
            "2008-01-12",
            "2008-05-04",
            "2008-06-28",
            "2008-08-16",
            "2008-09-13",
            "2008-12-27",
            "2009-01-10",
            "2009-04-18",
            "2009-07-18",
            "2009-08-15",
            "2009-09-12",
            "2009-12-26",
            "2010-04-10",
            "2010-07-17",
            "2010-08-21",
            "2010-09-11",
            "2010-09-18",
            "2011-03-12",
            "2011-04-30",
            "2011-07-09",
            "2011-08-06",
            "2011-08-20",
            "2011-09-17",
            "2012-03-24",
            "2012-04-14",
            "2012-04-28",
            "2012-07-14",
            "2012-07-21",
            "2012-08-18",
            "2012-09-29",
            "2013-04-13",
            "2013-05-18",
            "2013-06-01",
            "2013-07-13",
            "2013-08-24",
            "2013-09-28",
            "2013-12-28",
            "2014-02-01",
            "2014-04-26",
            "2014-05-31",
            "2014-08-02",
            "2014-08-23",
            "2014-09-27",
            "2014-12-20",
            "2014-12-27",
            "2015-04-11",
            "2015-04-18",
            "2015-07-18",
            "2015-09-19",
            "2016-03-12",
            "2016-04-02",
            "2016-09-24",
            "2017-05-06",
            "2017-05-20",
            "2017-09-23",
            "2018-03-17",
            "2018-04-07",
            "2018-05-05",
            "2018-10-27",
            "2018-11-03",
            "2020-02-01",
            "2020-05-23",
            "2021-09-25",
        )

        for year, dts in {
            2002: ("2002-12-29",),
            2006: (
                "2006-01-08",
                "2006-05-06",
                "2006-12-30",
            ),
            2007: (
                "2007-01-07",
                "2007-04-28",
                "2007-12-29",
            ),
            2008: (
                "2008-01-12",
                "2008-05-04",
                "2008-06-28",
                "2008-08-16",
                "2008-09-13",
                "2008-12-27",
            ),
            2009: (
                "2009-01-10",
                "2009-04-18",
                "2009-07-18",
                "2009-08-15",
                "2009-09-12",
                "2009-12-26",
            ),
            2014: (
                "2014-02-01",
                "2014-04-26",
                "2014-05-31",
                "2014-08-02",
                "2014-08-23",
                "2014-09-27",
                "2014-12-20",
                "2014-12-27",
            ),
        }.items():
            self.assertWorkingDay(Armenia(years=year), dts)

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2025-01-01", "Նոր տարվա օր"),
            ("2025-01-02", "Նոր տարվա օր"),
            ("2025-01-06", "Սուրբ Ծնունդ եւ Հայտնություն"),
            ("2025-01-28", "Բանակի օր"),
            ("2025-02-21", "Մայրենի լեզվի օր"),
            ("2025-02-27", "Սուրբ Վարդանանց տոն"),
            (
                "2025-02-28",
                "Ադրբեջանական ԽՍՀ-ում կազմակերպված ջարդերի զոհերի հիշատակի եւ բռնագաղթված հայ "
                "բնակչության իրավունքների պաշտպանության օր",
            ),
            ("2025-03-08", "Կանանց տոն"),
            ("2025-04-07", "Մայրության և գեղեցկության տոն"),
            ("2025-04-16", "Հայ կինոյի օր"),
            ("2025-04-18", "Հարկ վճարողի օր"),
            ("2025-04-24", "Հայոց ցեղասպանության զոհերի հիշատակի օր"),
            ("2025-04-26", "Հայաստանի Հանրապետության քաղաքացու օր"),
            ("2025-05-01", "Աշխատանքի օր"),
            ("2025-05-08", "Երկրապահի օր"),
            ("2025-05-09", "Հաղթանակի և Խաղաղության տոն"),
            ("2025-05-15", "Ընտանիքի օր"),
            ("2025-05-16", "Ուսանողների եւ երիտասարդների օր"),
            ("2025-05-28", "Հանրապետության օր"),
            ("2025-06-01", "Երեխաների իրավունքների պաշտպանության օր"),
            ("2025-06-14", "Բռնադատվածների հիշատակի օր"),
            ("2025-06-17", "Հայրերի օր"),
            ("2025-06-22", "Սուրբ Էջմիածնի տոն"),
            ("2025-07-05", "Պետական խորհրդանիշների օր; Սահմանադրության օր"),
            ("2025-08-03", "2014 թվականի՝ Սինջարի եզդիների ցեղասպանության զոհերի հիշատակի օր"),
            ("2025-09-01", "Գիտելիքի եւ դպրության օր"),
            ("2025-09-21", "Անկախության օր"),
            (
                "2025-10-05",
                "Հայաստանի Հանրապետության ազգային փոքրամասնությունների օր; Ուսուցչի օր",
            ),
            ("2025-10-11", "Թարգմանչաց տոն"),
            ("2025-11-10", "Տեղական ինքնակառավարման օր"),
            ("2025-12-07", "Երկրաշարժի զոհերի հիշատակի եւ աղետներին դիմակայունության օր"),
            ("2025-12-09", "Ցեղասպանությունների դատապարտման եւ կանխարգելման օր"),
            ("2025-12-31", "Նոր տարվա գիշեր"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2025-01-01", "New Year's Day"),
            ("2025-01-02", "New Year's Day"),
            ("2025-01-06", "Christmas and Epiphany"),
            ("2025-01-28", "Army Day"),
            ("2025-02-21", "Mother Language Day"),
            ("2025-02-27", "Saint Vardanants' Day"),
            (
                "2025-02-28",
                "Day of Remembrance of the Victims of the Massacres Organized in the Azerbaijan "
                "SSR and the Protection of the Rights of the Deported Armenian Population",
            ),
            ("2025-03-08", "Women's Day"),
            ("2025-04-07", "Motherhood and Beauty Day"),
            ("2025-04-16", "Armenian Cinema Day"),
            ("2025-04-18", "Taxpayer Day"),
            ("2025-04-24", "Day of Remembrance of the Victims of Armenian Genocide"),
            ("2025-04-26", "Citizen of the Republic of Armenia Day"),
            ("2025-05-01", "Labor Day"),
            ("2025-05-08", "Defender's Day"),
            ("2025-05-09", "Victory and Peace Day"),
            ("2025-05-15", "Family Day"),
            ("2025-05-16", "Students and Youth Day"),
            ("2025-05-28", "Republic Day"),
            ("2025-06-01", "Children's Rights Protection Day"),
            ("2025-06-14", "Day of Remembrance of the Repressed"),
            ("2025-06-17", "Father's Day"),
            ("2025-06-22", "Feast of Holy Etchmiadzin"),
            ("2025-07-05", "Constitution Day; State Symbols' Day"),
            (
                "2025-08-03",
                "Day of Remembrance of the Victims of the Sinjar Yazidi Genocide of 2014",
            ),
            ("2025-09-01", "Knowledge and Literacy Day"),
            ("2025-09-21", "Independence Day"),
            ("2025-10-05", "Day of National Minorities of the Republic of Armenia; Teacher's Day"),
            ("2025-10-11", "Translators' Day"),
            ("2025-11-10", "Local Self-Government Day"),
            ("2025-12-07", "Earthquake Victims Remembrance Day and Disaster Resilience Day"),
            ("2025-12-09", "Day of the Condemnation and Prevention of Genocides"),
            ("2025-12-31", "New Year's Eve"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

import re
import unicodedata
from pathlib import Path
from unittest import TestCase

from holidays import (
    country_holidays,
    financial_holidays,
    list_localized_countries,
    list_localized_financial,
    list_supported_countries,
    list_supported_financial,
)
from holidays.constants import PUBLIC


class TestAuthors(TestCase):
    @classmethod
    def setUpClass(cls):
        cls.content = Path("CONTRIBUTORS").read_text(encoding="UTF-8").strip().split("\n")
        super().setUpClass()

    def test_authors_list(self):
        authors = self.content[1:]
        self.assertEqual(
            authors,
            sorted(authors),
            "Contributors list should be sorted alphabetically.\n"
            + "\n".join((f"{c} != {s}" for c, s in zip(authors, sorted(authors)) if c != s)),
        )


class TestReadme(TestCase):
    @classmethod
    def setUpClass(cls):
        cls.readme_content = Path("README.md").read_text(encoding="UTF-8")

        super().setUpClass()

    def _parse_table(self, table_index: int):
        """Parse HTML table at given index from README.md content."""

        table_cell_re = re.compile(r"<td>(.*?)</td>")
        tables = re.findall(r"<table style.*?</table>", self.readme_content, re.DOTALL)  # type: ignore[attr-defined]
        table = tables[table_index]
        rows = re.findall(r"<tr>(.*?)</tr>", table, re.DOTALL)
        return [table_cell_re.findall(line) for line in rows[1:]]

    def test_supported_countries_count(self):
        actual_country_count = len(list_supported_countries(include_aliases=False))
        readme_country_count = int(
            re.findall(r"We currently support (\d+) country codes.", self.readme_content)[0]
        )
        self.assertEqual(
            readme_country_count,
            actual_country_count,
            "README.md supported countries statement is out of date: "
            f"'We currently support {readme_country_count} countries'. "
            f"Actual supported countries count: {actual_country_count}",
        )

    def test_supported_countries_table(self):
        country_alpha_2_codes = set()
        country_default_languages = {}
        country_names = []
        country_subdivisions = {}
        country_subdivisions_aliases = {}
        country_supported_languages = {}
        country_supported_categories = {}
        subdivision_group_re = re.compile(r"\w+:\s*([^\n:]+)")
        subdivision_and_aliases_re = re.compile(r",(?![^()]*\))")
        subdivision_aliases_re = re.compile(r"(.*?)\s\(([^()]*)\)")
        default_value_re = re.compile(r"<strong>(.*?)</strong>")

        # Parse 1st table.
        table_content = self._parse_table(0)

        for row in table_content:
            # Country: 1st column.
            country_names.append(
                re.sub(
                    r"[-,\s]",
                    "",
                    unicodedata.normalize("NFKD", row[0])
                    .encode("ascii", "ignore")
                    .decode("ascii"),
                ).lower()
            )

            # Code: 2nd column.
            country_code = row[1]
            if country_code:
                country_alpha_2_codes.add(country_code)

            # Subdivisions and their aliases: 3rd column.
            country_subdivisions[country_code] = []
            country_subdivisions_aliases[country_code] = {}
            subdivision_str = row[2]
            if subdivision_str:
                for subdivision_aliases_group in subdivision_str.split(";"):
                    # Exclude empty subdivisions.
                    if ":" not in subdivision_aliases_group:
                        country_subdivisions[country_code] = []
                        continue

                    # Combine all subdivision and their aliases.
                    subdivision_aliases_group = subdivision_group_re.findall(
                        subdivision_aliases_group
                    )[0]
                    for subdivision_aliases in subdivision_and_aliases_re.split(
                        subdivision_aliases_group
                    ):
                        if "(" in subdivision_aliases:  # Subdivision with aliases.
                            subdivision_aliases = subdivision_aliases_re.match(subdivision_aliases)
                            subdivision = subdivision_aliases.group(1)
                            aliases = subdivision_aliases.group(2).split(", ")
                            # "Virgin Islands, U.S." special case.
                            if len(aliases) == 2 and aliases[1] == "U.S.":
                                aliases = [subdivision_aliases.group(2)]
                        else:
                            aliases = []
                            subdivision = subdivision_aliases
                        subdivision = subdivision.strip()

                        country_subdivisions[country_code].append(subdivision)
                        country_subdivisions_aliases[country_code][subdivision] = aliases

            # Supported Languages: 4th column.
            supported_languages = row[3]
            if supported_languages:
                languages = []
                for supported_language in supported_languages.split(","):
                    supported_language = supported_language.strip()

                    if "<strong>" in supported_language:
                        supported_language = default_value_re.search(supported_language).group(1)
                        country_default_languages[country_code] = supported_language
                    languages.append(supported_language)

                country_supported_languages[country_code] = languages

            # Supported Categories: 5th column.
            supported_categories = row[4]
            if supported_categories:
                categories = [PUBLIC]
                for supported_category in supported_categories.split(","):
                    categories.append(supported_category.strip().lower())
                country_supported_categories[country_code] = sorted(categories)

        # Check the data.
        self.assertEqual(
            country_names,
            sorted(country_names),
            "The supported countries table must be sorted alphabetically by "
            "country name.\n"
            + "\n".join(
                (f"{c} != {s}" for c, s in zip(country_names, sorted(country_names)) if c != s)
            ),
        )

        supported_countries = list_supported_countries(include_aliases=False)
        localized_countries = list_localized_countries(include_aliases=False)
        for country_code in supported_countries:
            instance = country_holidays(country_code)
            country_name = instance.__class__.__base__.__name__

            # Make sure country alpha-2 code is shown correctly.
            self.assertIn(
                instance.country,
                country_alpha_2_codes,
                f"Country '{country_name}' alpha-2 code is not shown correctly in the table.",
            )

            # Make sure country subdivisions are shown correctly.
            self.assertEqual(
                supported_countries[country_code],
                country_subdivisions[country_code],
                f"Country '{country_name}' subdivisions are not shown "
                "correctly in the table.\n"
                + "\n".join(
                    (
                        f"{c} != {s}"
                        for c, s in zip(
                            supported_countries[country_code], country_subdivisions[country_code]
                        )
                        if c != s
                    )
                ),
            )

            # Make sure country subdivisions aliases are shown correctly.
            subdivision_aliases = instance.get_subdivision_aliases()
            for subdivision in instance.subdivisions:
                self.assertEqual(
                    subdivision_aliases.get(subdivision, []),
                    country_subdivisions_aliases[country_code][subdivision],
                    f"Country '{country_name}' subdivisions '{subdivision}' aliases are not shown "
                    "correctly in the table.\n",
                )

            # Make sure supported languages are shown correctly.
            if country_code in localized_countries:
                supported_languages = localized_countries[country_code]
                self.assertEqual(
                    supported_languages,
                    country_supported_languages.get(country_code),
                    f"Country {country_name} supported languages are not "
                    "shown correctly in the table. The column must contain "
                    "all supported languages: "
                    f"{', '.join(instance.supported_languages)}",
                )

                self.assertEqual(
                    country_default_languages.get(country_code),
                    instance.default_language,
                    f"Country {country_name} default language is not shown "
                    "correctly in the table. Use <strong>language</strong> format "
                    "to specify the country default language: "
                    f"<strong>{instance.default_language}</strong>.",
                )

            # Make sure supported categories are shown correctly.
            supported_categories = sorted(instance.supported_categories)
            self.assertEqual(
                supported_categories,
                country_supported_categories.get(country_code, [PUBLIC]),
                f"Country {country_name} supported categories are not "
                "shown correctly in the table. The column must contain "
                "all supported categories: "
                f"{', '.join(instance.supported_categories)}",
            )

    def test_supported_markets_table(self):
        market_mic_codes = set()
        market_default_languages = {}
        market_names = []
        market_supported_languages = {}
        market_supported_categories = {}
        default_value_re = re.compile(r"<strong>(.*?)</strong>")

        # Parse 2nd table.
        table_content = self._parse_table(1)
        replace_chars = str.maketrans(
            {
                " ": "",
                ",": "",
                "ã": "a",
            }
        )

        for row in table_content:
            # Market: 1st column.
            name = row[0].translate(replace_chars).lower()
            market_names.append(name)

            # Code: 2nd column.
            market_code = row[1]
            if market_code:
                market_mic_codes.add(market_code)

            # Supported Languages: 4th column.
            supported_languages = row[3]
            if supported_languages:
                languages = []
                for supported_language in supported_languages.split(","):
                    supported_language = supported_language.strip()

                    if "<strong>" in supported_language:
                        supported_language = default_value_re.search(supported_language).group(1)
                        market_default_languages[market_code] = supported_language
                    languages.append(supported_language)

                market_supported_languages[market_code] = languages

            # Supported Categories: 5th column.
            supported_categories = row[4]
            if supported_categories:
                categories = [PUBLIC]
                for supported_category in supported_categories.split(","):
                    categories.append(supported_category.strip().lower())
                market_supported_categories[market_code] = sorted(categories)

        # Check the data.
        self.assertEqual(
            market_names,
            sorted(market_names),
            "The supported markets table must be sorted alphabetically by market name.\n"
            + "\n".join(
                (f"{c} != {s}" for c, s in zip(market_names, sorted(market_names)) if c != s)
            ),
        )

        supported_markets = list_supported_financial(include_aliases=False)
        localized_markets = list_localized_financial(include_aliases=False)
        for market_code in supported_markets:
            instance = financial_holidays(market_code)
            market_name = instance.__class__.__base__.__name__

            # Make sure market name is shown correctly.
            self.assertIn(
                market_name.lower(),
                market_names,
                f"Market '{market_name}' name is not shown correctly in the table.",
            )

            # Make sure market MIC code is shown correctly.
            self.assertIn(
                instance.market,
                market_mic_codes,
                f"Market '{market_name}' MIC code is not shown correctly in the table.",
            )

            # Make sure supported languages are shown correctly.
            if market_code in localized_markets:
                supported_languages = localized_markets[market_code]
                self.assertEqual(
                    supported_languages,
                    market_supported_languages.get(market_code),
                    f"Market {market_name} supported languages are not "
                    "shown correctly in the table. The column must contain "
                    "all supported languages: "
                    f"{', '.join(instance.supported_languages)}",
                )

                self.assertEqual(
                    market_default_languages.get(market_code),
                    instance.default_language,
                    f"Market {market_name} default language is not shown "
                    "correctly in the table. Use <strong>language</strong> format "
                    "to specify the market default language: "
                    f"<strong>{instance.default_language}</strong>.",
                )

            # Make sure supported categories are shown correctly.
            supported_categories = sorted(instance.supported_categories)
            self.assertEqual(
                supported_categories,
                market_supported_categories.get(market_code, [PUBLIC]),
                f"Market {market_name} supported categories are not "
                "shown correctly in the table. The column must contain "
                "all supported categories: "
                f"{', '.join(instance.supported_categories)}",
            )
